# -*- Tcl -*-
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.IDENT     $Id: printMedia.tcl,v 1.1 2004/09/06 14:47:14 cguirao Exp $
#.LANGUAGE  Tcl/Tk
#.AUTHOR    Fabio Sogni, ESO
#.PURPOSE   Script to print media automatically
#.COMMENTS 
#   Take a media, mount it in a writer, read the volume than
#   print a label containing that volume
#
#.VERSION   1.0  10-Oct-00: FS, Creation
#--------------------------------------------------------------------

#Initialize the server, reset the robot and count the CDs
proc printMediaProcInitialize {} {
    global gvar
    
    package require Tclx 

    Debug  "Define variables" 1
    printMediaProcDefineVar

    Debug "Check if romanic is running." 1
    checkRomanic

    Debug "Set options" 1
    romanicSetOptions
   
    Debug "Read configuration file" 1


    #Read the configuration file only in automatic mode,
    #otherwise get the parameter from the command line.
    if {$gvar(automatic)} {
	printMediaReadConfFile 
    } 

    printMediaProcInitializeLoop
    
}

#End the initialization
proc printMediaProcInitializeLoop {} {
    global gvar

    set gvar(cancelGlobLoop) 0
    set gvar(cancelServeRequestsLoop) 0
    
    #Open and configure the serial ports.
    openPorts   
    
    #If we are in automatic means that there is a robot connected.
    #Reset the robot
    if {$gvar(automatic)} {
	Debug  "Get sense" 1
	romanicRequestSense
	Debug "Reset robot" 1
	romanicResetRobot
    }
    
    #Print the volume.
    printMedia
}


#Define some constants and initialize the variables.
proc printMediaProcDefineVar {} {
    global gvar devices
    
    set gvar(requests) ""
    set gvar(automatic) 0
    set gvar(reqDir)   /tmp
    
    #set gvar(inputMagazine)     1
    #set gvar(outputMagazine)    3

    set gvar(enquiryString)     "\x1B\x21\x65\x30\x43"
    set gvar(requestSenseString) "\x1B\x21\x65\x31\x43"
    set gvar(moveArmString)      "\x1B\x21\x66\x31\x32\x30\x30\x32\x43"

    set gvar(escape)            "\x1B\x21"
    set gvar(header1)           "\x66\x31"
    set gvar(trail)             "\x43"

    #Addresses
    set gvar(blankAddress)      "\x32\x30\x30"; #blank tray address
    set gvar(writtenAddress)    "\x32\x30\x31"; #written tray address
    set gvar(badAddress)        "\x32\x30\x32"; #bad tray address
    set gvar(cdAddress)         "\x32\x34\x30"; #cd address
    set gvar(printerAddress)    "\x32\x36\x30"; #printerAddress

    #Actions
    set gvar(takeMedia)            "\x32"; #take media from selected tray
    set gvar(putMedia)             "\x31"; #put media from selected tray
    set gvar(noAction)          "\x30"; #move arm on top
    set gvar(openPrinter)       "\x1B\x21\x66\x32\x32\x36\x30\x31\x43"; #open printer
    set gvar(closePrinter)      "\x1B\x21\x66\x32\x32\x36\x30\x30\x43"; #close printer
    set gvar(fanOn)             "\x1B\x21\x65\x32\x31\x43\x00"; #Cooling on
    set gvar(fanOff)            "\x1B\x21\x65\x32\x30\x43\x00"; #Cooling off

    set gvar(ttyList) "";    # list containing all the tty     
    set gvar(deviceList) ""; # list containing all the devices
    set gvar(speedList)  ""; # list containing the speed of the ttys 
    set gvar(badLabel) ""  ; # label to use in case the writing fails.
}

#Print a medium.
proc printMedia {} {
    global gvar

    if {$gvar(automatic)} {
	while {$gvar(numMedia) > 0} {
	    printMediaAutomatic
	    incr gvar(numMedia) -1
	}
    } else {
	printLabel goodLabel $gvar(volumeName)
    } 
}


#Take medium from magazine 1, read it and print the label.
proc printMediaAutomatic {} {
    global gvar


    set tty $gvar(tty)
    set ttyTail [file tail $tty]
    set gvar(id) printMedia
     
    #Get a disk from magazine n. 1 
    getBlank 
    ejectCD $gvar(device)
    putMedia
    
    while {[loadCD $gvar(device)] == "error"} {
	ejectCD $gvar(device)
	#puts "waitForFixError"
	puts "I cannot read the media in $gvar(device)\nPlease insert a valid media."
	}
    
    
    #Mount here the disk.
    mountDisk
    
    #Get the volume name
    set volumeName [getCDname]

    #Unmount the disk
    unmountDisk

    #Put a medium in the printer and print it.
    ejectCD $gvar(device)
    getMedia
    loadCD $gvar(device)
    if {$volumeName != "error"} {
        openPrinter  
	putPrinter   
	closePrinter 
	printLabel goodLabel $volumeName
	openPrinter  
	getPrinter   
	closePrinter 
	putWritten 
    } else {
	putBad
    }
    armOnTop 
}


#Enable the event handler. Not used.
proc mainLoop {} {

    fileevent stdin readable fakeProc
    vwait forever    
}

#Trick to enable the event handler. Not used
proc fakeProc {} {return}

#################### Main ###############

proc main {argv} {
    global gvar 

    set gvar(args) $argv
    printMediaProcInitialize
}

main $argv


##########################################
