#Procedure file containing functions to pilot the Rimage robot.


#Define the robot commands for the rimage robot.
proc romanicDefineCommandsRimage {} {
	global gvar

    set gvar(enquiryString)     "\x1B\x21\x65\x30\x43"
    set gvar(requestSenseString) "\x1B\x21\x65\x31\x43"
    set gvar(moveArmString)      "\x1B\x21\x66\x31\x32\x30\x30\x32\x43"

    set gvar(escape)            "\x1B\x21"
    set gvar(header1)           "\x66\x31"
    set gvar(trail)             "\x43"

    #Addresses
    set gvar(blankAddress)      "\x32\x30\x30"; #blank tray address
    set gvar(writtenAddress)    "\x32\x30\x31"; #written tray address
    set gvar(badAddress)        "\x32\x30\x32"; #bad tray address
    set gvar(cdAddress)         "\x32\x34\x30"; #cd address
    set gvar(printerAddress)    "\x32\x36\x30"; #printerAddress

    #Actions
    set gvar(takeMedia)            "\x32"; #take media from selected tray
    set gvar(putMedia)             "\x31"; #put media from selected tray
    set gvar(noAction)          "\x30"; #move arm on top
    set gvar(openPrinter)       "\x1B\x21\x66\x32\x32\x36\x30\x31\x43"; #open printer
    set gvar(closePrinter)      "\x1B\x21\x66\x32\x32\x36\x30\x30\x43"; #close printer
    set gvar(fanOn)             "\x1B\x21\x65\x32\x31\x43\x00"; #Cooling on
    set gvar(fanOff)            "\x1B\x21\x65\x32\x30\x43\x00"; #Cooling off
}


#Low level command to open the printer.
proc openPrinterRimage {} {
	global gvar 
	
    Log "Open Printer Rimage"
    sendMessage $gvar(openPrinter)
    readPort 6
}

#Low level commando to close the printer.
proc closePrinterRimage {} {
    global gvar 
    
    Log "Close Printer"
    sendMessage $gvar(closePrinter)
    readPort 6
}


#Switch on the fan.
proc fanOnRimage {} {
    global gvar

    write_file $gvar(reqDir)/fan-$gvar(id)-$gvar(ttyTail).romanic ""
    Log "Switch on fan"
    sendMessage $gvar(fanOn)
    readPort 6 
}

#Switch off the fan.
proc fanOffRimage {} {
    global gvar

    Log "Switch off fan"
    sendMessage $gvar(fanOff)
    readPort 6
}

#Retrieve the enquiry string from the Rimage robot.
proc romanicPilotEnquiryRimage {} {
    global gvar

    Debug "Enquiry" 3
    sendMessage $gvar(enquiryString)
    readPort 6
}

#Pick up a medium from the specified magazine.
proc pickUpCDRimage {tty magazine} {
    global gvar
    
    Log "Pick up CD"
    switch $magazine {
	"blank"    {positionMedium  $gvar(blankAddress)    $gvar(takeMedia)}
	"written"  {positionMedium  $gvar(writtenAddress)  $gvar(takeMedia)}
	"bad"      {positionMedium  $gvar(badAddress)      $gvar(takeMedia)}
    }
}

#Put a medium in the specified magazine.
proc dropCDRimage {tty magazine} {
    global gvar 

    Log "Drop CD"
    switch $magazine {
	"blank"    {positionMedium $gvar(blankAddress)   $gvar(putMedia)}
	"written"  {positionMedium $gvar(writtenAddress) $gvar(putMedia)}
	"bad"      {positionMedium $gvar(badAddress)     $gvar(putMedia)}
    }
}

#Count the media in the specified magazine.
proc readElementRimage {tty magazine} {
    global gvar
    
    switch $magazine {
	"blank"    {set address "\x32\x30\x30"}
	"written"  {set address "\x32\x30\x31"}
	"bad"      {set address "\x32\x30\x32"}
	"arm"      {set address "\x32\x32\x30"}
    }

    sendMessage "\x1B\x21\x66\x30$address\x43"
    set returnValue [readPort 12]
    return $returnValue
}

#Enquiry the robot if the previous command gave an error.
#It gives more information about an error.
proc romanicRequestSenseRimage {} {
    global gvar 

    sendMessage $gvar(requestSenseString)
    set sense [readPort 11]
    set senseData1 [string range $sense 4 6]
    set senseData2 [string range $sense 7 9]

    Debug "sense: $sense" 3
    Debug "Sense data 1: $senseData1" 3
    Debug "Sense data 2: $senseData2" 3
    
    switch  $senseData1 {
	"000" {set sense1 "Ok"}
	"001" {set sense1 "Recovered Error (Retry was required)"}
	"002" {set sense1 "Not Ready; Check Condition. The unit was powered-on"}
	"003" {set sense1 "Medium Error."}
	"004" {set sense1 "Medium Hardware Error. Indicates the unit had problems gripping the media or the media is not present. Perform request element status for more information."}
	"005" {set sense1 "Illegal Request. Indicates there is an illegal parameter in the command."}
	"006" {set sense1 "Unit Attention. Indicates the operator access is currently open or was opened during operation."}
	"007" {set sense1 "No Recorder Detected. Indicates the Drawer Detect sensor could not locate an open recorder drawer."}
	"008" {set sense1 "No carousel."}
	"009" {set sense1 "No Printer or Printer Communication Failure. Cannot communicate with the printer."}
	"010" {set sense1 "Carousel Failure. Cannot position the carousel."}
	"011" {set sense1 "Lif Failure. Cannot position the lift."}
	"012" {set sense1 "Drawer Detect sensor is not functioning."}
	"013" {set sense1 "Reserved."}
	"014" {set sense1 "Reserved."}
	"015" {set sense1 "Host Communication Failure."}
	"016" {set sense1 "Program Flash failure."}
	"017" {set sense1 "Rotate medium failure."}
	"018" {set sense1 "Mailbox Failure."}
	"019" {set sense1 "Perfect Print Failure."}
	default { set sense1 "Wrong Sense." }
    }

    switch $senseData2 {
	"000" {set sense2 "Ok"}
	"001" {set sense2 "Ribbon out."}
	"002" {set sense2 "Busy Printing Image (the image is being printed)."}
	"003" {set sense2 "Lower Print Head Failure."}
	"004" {set sense2 "Raise Print Head Failure."}
	"005" {set sense2 "Position Printer Drawer Home (out) Failure."}
	"006" {set sense2 "Position Printer Drawer In Failure."}
	"007" {set sense2 "Position Printer Drawer Out Failure."}
	"008" {set sense2 "Corrupt/Invalid Data in Command."}
	"009" {set sense2 "Data Mismatch."}
	"010" {set sense2 "Left Margin Length Invalid."}
	"011" {set sense2 "No Margin Specified When RLE is Disabled."}
	"012" {set sense2 "CD Detection - A print command was received without a disc being present."}
	"013" {set sense2 "Color change Error - The printer is unable to detect the requested ribbon Color."}
	default { set sense2 "Wrong Sense." }
    }

    Debug  "Codes: $sense1 --- $sense2" 3
    #puts "Request sense returns: $sense1 --- $sense2"
    return "$sense1 --- $sense2"
}


#Get a disk from magazine 1
proc getBlankRimage {} {
    global gvar 
    
    Log "getBlank from Rimage"
    set cmd "positionMedium $gvar(blankAddress) $gvar(takeMedia)"
    set result [loopCommand $cmd 3]
    
    if {$result != "ok"} {
	switch -glob $result {
	    "Medium Error.*"    {waitForFixError "Blank magazine of robot $gvar(ttyTail) is empty"}
	    "Unit Attention.*" {
		waitForFixError "Door open or robot $gvar(ttyTail) powered off"
	    }
	} 
	getBlankRimage
    }
}

#Get a disk from magazine 2
proc getWrittenRimage {} {
    global gvar 

    Log "getWritten" 
    set cmd "positionMedium  $gvar(writtenAddress) $gvar(takeMedia)"
    set result [loopCommand $cmd 3]
    
    if {$result != "ok"} {
	switch -glob $result {
	    "Medium Error.*"    {waitForFixError "Written magazine of robot $gvar(ttyTail) is empty"}
	    "Unit Attention.*" {
		waitForFixError "Door open or robot $gvar(ttyTail) powered off"
	    }
	} 
	getWritten
    }
}

#Get a disk from magazine 3
proc getBadRimage {} {
    global gvar 

    Log "getBad"
    set cmd "positionMedium $gvar(badAddress) $gvar(takeMedia)"
    set result [loopCommand $cmd 3]
    
    if {$result != "ok"} {
	switch -glob $result {
	    "Medium Error.*"    {waitForFixError "Bad magazine of robot $gvar(ttyTail) is empty"}
	    "Unit Attention.*" {
		waitForFixError "Door open or robot $gvar(ttyTail) powered off"
	    }
	} 
	getBad
    }
}


#Take a medium from the writer.
proc getMediaRimage {} {
    global gvar 
    
    Log "getMedia"
    set cmd "positionMedium $gvar(cdAddress) $gvar(takeMedia)"
    set result [loopCommand $cmd 3]
 
    if {$result != "ok"} {
	switch -glob $result {
	    "Medium Error.*"    {waitForFixError "Error while getting medium from the writer"}
	    "Unit Attention.*" {
		waitForFixError "Door open or robot $gvar(ttyTail) powered off"
	    }
	}
	armOnTop
	getMedia
    }   
}

#Get a disc from the input magazine.
#The variable gvar(inputMagazine) defines which one is considered input.
proc getInputCdRimage {} {
    global gvar
    
    set gvar(inputMagazine) [read_file -nonewline $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic]
    
    switch $gvar(inputMagazine) {
	"1" {set result [positionMedium $gvar(blankAddress) $gvar(takeMedia)]
	    set gvar(outputMagazine) 3
	}
	"2" {set result [positionMedium $gvar(writtenAddress) $gvar(takeMedia)]
	    set gvar(outputMagazine) 1
	}
    }
    
    if {$result != ""} {
	if {$gvar(inputMagazine) == 1} { 
	    Log "Switch to second magazine"
	    set gvar(inputMagazine)  2
	    set gvar(outputMagazine) 1
	    write_file $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic $gvar(inputMagazine)
	    getInputCd 
	} else {
	    switch -glob $result {
		"Medium Error.*"    {waitForFixError "Magazine 2\nOf robot $gvar(ttyTail) is empty.\nInsert new CDs in magazine\n1 AND 2"}
		"Unit Attention.*" {
		    waitForFixError "Door open or robot $gvar(ttyTail) powered off"
		    romanicRequestSense
		}
	    }
	    set gvar(inputMagazine)  1
	    set gvar(outputMagazine) 3
	    write_file $gvar(reqDir)/currentMagazine-$gvar(ttyTail).romanic $gvar(inputMagazine)
	    getInputCd
	}
    }
}

#Put a disc to the input magazine.
#The variable gvar(outputMagazine) defines which one is considered output.
proc putOutputCdRimage {} {
    global gvar

    Log "Put CD to magazine: $gvar(outputMagazine)"
    
    switch $gvar(outputMagazine) {
	"3" {set result [positionMedium $gvar(badAddress) $gvar(putMedia)]}
	"1" {set result [positionMedium $gvar(blankAddress) $gvar(putMedia)]}
    }
    
    if {$result != ""} {
	Log "Error putting CD to magazine $gvar(outputMagazine)"
	Log "Reason: $result"
	waitForFixError "Error putting CD to magazine $gvar(outputMagazine)\nof robot $gvar(ttyTail)"
	romanicRequestSense
	putOutputCd
    }
}

#Put a disc in the specified writer.
proc putDiskInWriterRimage {writer} {
    global gvar 
    
    ejectCD $gvar($gvar(id)-writerAddress)
    putMedia
    while {[loadCD $gvar($gvar(id)-writerAddress)] == "error"} {
	ejectCD $gvar($gvar(id)-writerAddress)
	waitForFixError "I cannot read the media in $gvar($gvar(id)-writerAddress)\nPlease insert a valid media."
    }
}

#Take a disc from the specified writer.
proc getDiskFromWriterRimage {writer} {
    global gvar 
    
    ejectCD $gvar($gvar(id)-writerAddress)
    getMedia
    loadCD $gvar($gvar(id)-writerAddress)
}


#Put a disk in a writer
proc putMediaRimage {} {
    global gvar

    Log "put CD to writer $gvar(cdAddress)"
    #set cmd "positionMedium $gvar(cdAddress) $gvar(putMedia)"
    #set result [loopCommand $cmd 3]
    
    #if {$result != "ok"} {
	#Log "Error putting CD to writer $gvar(cdAddress) of $gvar(tty) robot."
	#Log "Reason: $result"
	#waitForFixError "Error putting CD to writer\nof robot $gvar(ttyTail)."
    #}

    set result [ positionMedium $gvar(cdAddress) $gvar(putMedia) ]

    if {$result != ""} {
	waitForFixError "Error putting CD to writer of robot $gvar(ttyTail)"
	romanicRequestSense
	putMedia
    }
}


#Put a disc in the blank magazine.
proc putBlankRimage {} {
    global gvar 

    Log "putBlank"
    set cmd "positionMedium $gvar(blankAddress) $gvar(putMedia)"
    set result [loopCommand $cmd 3]
    
    if {$result != "ok"} {
	Log "Error putting CD to blank magazine of $gvar(tty) robot." 
	Log "Reason: $result" 
	waitForFixError "Error putting CD to blank magazine\nof robot $gvar(ttyTail)"
	putBlank
    }
}

#Put a disc in the written magazine.
proc putWrittenRimage {} {
    global gvar 

    Log "putWritten"
    set cmd  "positionMedium $gvar(writtenAddress) $gvar(putMedia)"
    set result [loopCommand $cmd 3]
    
    if {$result != "ok"} {
	Log "Error putting CD to written magazine of $gvar(tty) robot."
	Log "Reason: $result"
	waitForFixError "Error putting CD to written magazine\nof robot $gvar(ttyTail)."
	romanicRequestSense
	putWritten
    }
}

#Put a disk in the bad magazine.
proc putBadRimage {} {
    global gvar 

    Log "putBad"
    set cmd  "positionMedium $gvar(badAddress) $gvar(putMedia)"
    set result [loopCommand $cmd 3]
    
    if {$result != "ok"} {
	Log "Error putting CD to bad magazine of $gvar(tty) robot."
	Log "Reason: $result"
	waitForFixError "Error putting CD to bad magazine\nof robot $gvar(ttyTail)."
	romanicRequestSense
	putBad
    }
}

#Open tray, take a disc from the printer, close tray.
proc getDiskFromPrinterRimage {} {

    openPrinter  
    getPrinterRimage   
    closePrinter 
}


#This is used to get a disk from the printer in the
#initialization phase.
proc grabDiskFromPrinterRimage {} {
    global gvar
    
    set result [ positionMedium $gvar(printerAddress) $gvar(takeMedia) ]
    return $result
}


#Take a disc from the printer.
proc getPrinterRimage {} {
    global gvar 
    
    Log "getPrinter Rimage"
    set result [ loopCommand "positionMedium $gvar(printerAddress) $gvar(takeMedia)" 3 ]
    
    if {$result != "ok"} { 
	#if {![info exists gvar(id)]} { return 1 }
	Log "Error taking CD from printer of $gvar(tty) robot."
	Log "Reason: $result"
	waitForFixError "Error taking CD from printer\nof robot $gvar(tty)."
	getPrinter
    }
}

#Open tray, put a disc in the printer, close tray.
proc putDiskInPrinterRimage {} {
    
    openPrinter  
    putPrinterRimage   
    closePrinter  
}

#Put a disc in the printer.
proc putPrinterRimage {} {
    global gvar 
    
    Log "putPrinter" 
    set cmd  "positionMedium $gvar(printerAddress) $gvar(putMedia)"
    set result [loopCommand $cmd 3]
    
    if {$result != "ok"} {
	Log "Error putting CD to printer of $gvar(tty) robot."
	Log "Reason: $result"
	waitForFixError "Error putting CD to printer\nof robot $gvar(ttyTail)."
	putPrinterRimage
    }
}

#Send a file to the parallel port.
proc printLabelRimage {file} {
    global gvar 
    
    catch {exec cat $file > $gvar(tty)}
}


#Put the arm in the parking position.
proc armOnTopRimage {} {
    global gvar 

    Log "Arm on top"
    set cmd "positionMedium $gvar(printerAddress) $gvar(noAction)"
    set result [loopCommand $cmd 3]
    
    if {$result != "ok"} {
	Log "Error putting arm on top of $gvar(tty) robot."
	Log "Reason: $result"
	waitForFixError "Error putting arm on top of robot $gvar(tty)"
	romanicRequestSense
	armOnTop
    }
}

#Send low level commands to the robot.
proc positionMediumRimage {address action} {
    global gvar
    
    Debug "Position medium: $gvar(tty) $address $action" 1
    set commandString "$gvar(escape)$gvar(header1)$address$action$gvar(trail)"
    set expectedAnswer "\x1B\x21\x66\x31\x31\x43"
    
    sendMessage $commandString
    set status [readPort 6]
    if {$status != $expectedAnswer} {
	Debug "I expected $expectedAnswer" 1
	Debug "But got: $status" 1
	set requestSense [romanicRequestSense]
	Debug "And the sense is: $requestSense" 1
	return $requestSense
    }
    return ""
}


#Remove discs from the writers. Used at the initialization.
proc checkWritersRimage {tty} {
    global gvar
    
    foreach cd $gvar(deviceList) {
	armOnTop
	ejectCD $cd
	getMedia
	loadCD $cd
	if {[checkArm $tty] == 1} {putBad}
    }
}

#Return the number of CDs in a particular magazine in
#of a particular robot (RIMAGE)
proc checkMagazineRimage {tty magazine} {

    pickUpCD $tty $magazine
    dropCD   $tty $magazine
    set returnValue [readElement    $tty $magazine]
    set value       [string range   $returnValue 6 10]
    set numCD       [calculateValue $value]
    
    return $numCD    
}

#Check if there is a CD in the arm.
proc checkArmRimage {tty} {
    
    set returnValue [readElement $tty arm]
    set value       [string range $returnValue 5 5]
    return $value
    
    #Only for debug
    switch $value {
	"0" {puts "The arm is empty"}
	"1" {puts "The arm is busy" }
	"2" {puts "No arm"}
	"3" {puts "Arm status undefined"}
    }
}


#Free the picker in case there is a disc.
proc freeArmRimage {tty} {
    global gvar
    
    armOnTop
    if {[checkArm $tty] == 1} {
	putBad
    }
}

#Remove discs from the printer, used at the initialization.
proc checkPrinterRimage {tty} {
    global gvar

    armOnTop
    openPrinter
    grabDiskFromPrinterRimage
    closePrinter
    freeArmRimage $tty
    armOnTop
}

#Not used.
proc calibrateArmRimage {tty} { return }

#Executes a command for $attempts times and in case it
#failed all the times, gives an error.
proc loopCommandRimage {cmd attempts} {
    global gvar 
    
    Debug "Executing command: $cmd with $attempts attempts" 2
    set result [$cmd]
    
    if {$result != ""} {
	set inc 0
        while {$inc < $attempts && $result != "" } {
	    incr inc
            set result [ $cmd ]
        }
		
        if {$inc == $attempts} {
	    Debug "Failed executing $cmd after $attempts attempts" 2
	    return $result
        } else {
	    return "ok"
	}
    } else {
	return "ok"
    }
}



##########################################
