package org.eso.dfs.util;

/**
 * @author nkornwei
 *
 * A token containing keyword data read by a FitsTokenizer and passed to a KeywordTokenListener.
 */
public class KeywordToken {

    private String name = null;
    private String value = null;
    private int extension;
    private int indexInExtension;

    /**
     * Constructor for KeywordToken.
     */
    public KeywordToken() {
        // nothing to do.
    };
    
    public KeywordToken(String n, String v, int x, int idx) {
        name = n;
        value = v;
        extension = x;
        indexInExtension = idx;
    }

    /**
     * Returns the name.
     * @return String
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the value.
     * @return String
     */
    public String getValue() {
        return value;
    }

    /**
     * Sets the name.
     * @param name The name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Sets the value.
     * @param value The value to set
     */
    public void setValue(String value) {
        this.value = value;
    }

    /**
     * Returns the extension.
     * The extension in which the keyword is found,
     * 0 is the primary header.
     * @return int
     */
    public int getExtension() {
        return extension;
    }

    /**
     * Returns the indexInExtension.
     * The position of the keyword in the extension (from 0...).
     * @return int
     */
    public int getIndexInExtension() {
        return indexInExtension;
    }

    /**
     * Sets the extension.
     * The extension in which the keyword is found,
     * 0 is the primary header.
     * @param extension The extension to set
     */
    public void setExtension(int extension) {
        this.extension = extension;
    }

    /**
     * Sets the indexInExtension.
     * @param indexInExtension The indexInExtension to set
     * The position of the keyword in the extension (from 0...).
     */
    public void setIndexInExtension(int indexInExtension) {
        this.indexInExtension = indexInExtension;
    }
}
