package org.eso.gasgano.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Rectangle;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTable;

import org.eso.gasgano.tools.Exec;

/**
 * @author nkornwei
 *
 * Unrelated set of GUI functions.
 * 
 * TODO: make all public members accessible through getters. 
 */
public class GUIUtils {

    // Colors
    public static Color dataBaseColor = new Color(0.1f, 0.3f, 0.1f);

    private static final String iconbase = "/org/eso/gasgano/gui/icons/";

    // image addresses.
    public static final String dataGif     = iconbase + "data.gif";
    public static final String redGif      = iconbase + "redfits.gif";
    public static final String rawGif      = iconbase + "rawfits.gif";
    public static final String tfitsGif    = iconbase + "tfits.gif";
    public static final String hrglasGif   = iconbase + "ib_hourglas.gif";
    public static final String readGif     = iconbase + "readfits.gif";
    public static final String videoGif    = iconbase + "video.gif";
    public static final String documentGif = iconbase + "document.gif";
    public static final String disknavGif  = iconbase + "kdisknav.gif";
    public static final String gasganoGif  = iconbase + "Gasgano2.gif";
    public static final String podraceJpg  = iconbase + "podrace_bg.jpg";

    // labels for the tree icons

    public static final String obNodeIcon      = iconbase + "obFolder.gif";
    public static final String interNodeIcon   = iconbase + "interFolder.gif";
    public static final String defaultNodeIcon = iconbase + "folder.gif";
    public static final String programNodeIcon = iconbase + "pgFolder.gif";
    public static final String telNodeIcon     = iconbase + "telFolder.gif";

    public static JLabel obLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(obNodeIcon)),
            (int) JLabel.LEFT_ALIGNMENT);
    public static JLabel interLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(interNodeIcon)),
            (int) JLabel.LEFT_ALIGNMENT);
    public static JLabel defaultLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(defaultNodeIcon)),
            (int) JLabel.LEFT_ALIGNMENT);
    public static JLabel programLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(programNodeIcon)),
            (int) JLabel.LEFT_ALIGNMENT);
    public static JLabel telLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(telNodeIcon)),
            (int) JLabel.LEFT_ALIGNMENT);

    // serious images 

    public static JLabel dataLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(dataGif)),
            (int) JLabel.LEFT_ALIGNMENT);

    public static JLabel redLabel =
        new JLabel(
            new ImageIcon(Object.class.getResource(redGif)),
            (int) JLabel.LEFT_ALIGNMENT);
            
    // funny images

    public final static JLabel gasgLabel1 =
        new JLabel(new ImageIcon(Object.class.getResource(iconbase+"gasgano.gif")));

    public final static JLabel gasgLabel2 =
        new JLabel(
            new ImageIcon(Object.class.getResource(iconbase+"Gasgano2.gif")));

    public final static JLabel gasgLabel3 =
        new JLabel(
            new ImageIcon(Object.class.getResource(iconbase+"bg.esologo.gif")));


    /**
     * Convert a string description of a Rectangle into a Rectangle.
     * @param tmp String containing coordinates of rect.
     * @return Rectangle parsed from string, else null on parse error.
     */
    public static Rectangle stringToRect(String tmp) {
        Rectangle rc = null;
        int x, y, w, h = 0;
        int i1, i2, i3, i4 = 0;

        i1 = tmp.indexOf(",");
        i2 = tmp.indexOf(",", i1 + 1);
        i3 = tmp.indexOf(",", i2 + 1);
        if (i1 == -1 || i2 == -1 || i3 == -1) {
            return rc;
        }
        // further checks should be added here
        x = Integer.parseInt(tmp.substring(0, i1));
        y = Integer.parseInt(tmp.substring(i1 + 1, i2));
        w = Integer.parseInt(tmp.substring(i2 + 1, i3));
        h = Integer.parseInt(tmp.substring(i3 + 1, tmp.length()));
        rc = new Rectangle(x, y, w, h);
        //System.out.println(x +" "+y+" "+w+" "+h);
        return rc;
    }

    /**
     * Convert a Rectangle into a string description of a Rectangle
     * @param tmp Rectangle to convert to String.
     * @return String description of the Rectangle.
     */
    public static String rectToString(Rectangle tmp) {
        String rc = null;
        if (tmp == null) {
            return rc;
        }
        rc =
            new String(
                tmp.x + "," + tmp.y + "," + tmp.width + "," + tmp.height);
        return rc;
    }

    /**
     * this method differs from the Application.runCommand 
     * one only in that it sets the Cursor for the GUI
     */
    public static int runCommandInGui(
        Component comp,
        String command,
        int okResult) {
        Cursor c = comp.getCursor();
        int rc = 0;
        comp.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        rc = Exec.runCommand(command);
        comp.setCursor(c);
        if (rc != okResult) {
            userMessage(
                comp,
                "Error: "
                    + rc
                    + " running command:\n"
                    + Exec.cleverProcessCommand(command));
            System.err.println(
                "Error: " + rc + " running command:\n" + command + "\n");
        }
        return rc;
    }

    /** 
     * @param comp
     * @param msg
     * Display a message to the user.
     */
    public static void userMessage(Component comp, Object msg) {
        JOptionPane.showMessageDialog(comp, msg);
    }
    
    public static void userMessage(Component comp, String title, Object msg) {
        JOptionPane.showMessageDialog(comp, msg, title, JOptionPane.INFORMATION_MESSAGE);
    }
    
    /** 
     * @param comp
     * @param msg
     * Display an error message to the user.
     */
    public static void userError(Component comp, Object msg) {
        JOptionPane.showMessageDialog(comp, msg, "Error", JOptionPane.ERROR_MESSAGE);
    }
    
    /**
     * return the base path to the icons for gasgano.
     * @return String
     */
    public static String getIconbase() {
        return iconbase;
    }
    
    /**
     * Center the target frame in the middle of the reference frame.
     * @param ref
     * @param targ
     */
    public static void center(Component ref, JFrame targ) {
        targ.setLocationRelativeTo(ref);
    }
    
    /**
     * Stop the editing on the specified table.
     * @param tbl
     */
    public static void stopEditing(JTable table) {
        int row = table.getEditingRow();
        int col = table.getEditingColumn();
        if (row >= 0 && row < table.getRowCount() && col >= 0 && col < table.getColumnCount()) {
            table.getCellEditor(row,col).stopCellEditing(); 
        }
    }
}
