package org.eso.ohs.core.dbb.htmlclient;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * This is an implementation of the DbbHtmlWidget abstract class,
 * defining a Multi Combo box widget. Multi selection as the name implies
 * allows the user to select more than on item using the standard mouse
 * action or the CTRL-Button  
 * <br>
 * @author amchavan, ESO 09-Dec-2003
 * @version $Revision: 1.4 $
 */

// $Id: DbbHtmlMultiBox.java,v 1.4 2004/04/14 12:45:41 janwunah Exp $

public class DbbHtmlMultiBox extends DbbHtmlWidget {

	public static final String cvsID_ =
		"$Id: DbbHtmlMultiBox.java,v 1.4 2004/04/14 12:45:41 janwunah Exp $";

	/** Array of database objects. */
	private Object[] dbArray_;

	/**
	 * Create a DbbHtmlComboBox, with a specific label, associated with a
	 * specific chunk
	 *
	 * @param labeltext Label text to be displayed.
	 * @param chunk     DbbSqlChunk object.
	 */
	public DbbHtmlMultiBox(String labeltext, DbbSqlChunk chunk) {
		super(labeltext, chunk);
		setDefault("");
	}

	public DbbHtmlMultiBox(
		String labletext,
		DbbSqlChunk chunk,
		Object[] items) {
		this(labletext, chunk);
		dbArray_ = items;
	}

	/** Implementation of the abstract method */
	public String render(int indentLevel) {
		int size = dbArray_.length + 1;

		String name = getName();
		StringBuffer sb = new StringBuffer();

		sb
			.append(indentSpaces(indentLevel))
			.append("<td>")
			.append(getLabel())
			.append("</td>\n")
			.append(indentSpaces(indentLevel))
			.append("<td>")
			.append("<select name=")
			.append(name)
			.append(" multiple size=")
			.append(size)
			.append("\">\n");

		//The first option is always an empty String
		//<option value="valX">valX
		sb
			.append("<option value=\"")
			.append("\"Empty\"")
			.append("\">")
			.append(" ")
			.append("\n");

		if (dbArray_.length > 0) {
			for (int i = 0; i < dbArray_.length; i++) {
				String value = (String) dbArray_[i];
				sb.append("<option value=\"").append(value).append(
					"\">").append(
					value).append(
					"\n");
			}
		}
		sb.append("</select>\n").append("</td>\n");

		return sb.toString();
	}

	/**
	 * Set the widget's value (implementation of the abstract method)
	 * @param val is the selected value in the combo box
	 */
	public void set(String val) {
		boolean active = false;
		stdlog_.debug("Set multibox to value "+val);
		if (val != null && val.length() > 0) {
			active = true;
		}
		DbbSqlChunk c = getSqlChunk();
		c.setActive(active);
		c.setValue(val);
		value_ = val;

	}
}
