package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * A Document model to only permit the input of a declination value.
 * 
 * <p>
 * $Id: DeclinationModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 * @author cmavros 29-10-01 Add insertString()
 */

public class DeclinationModel extends InputMaskModel {

    private final float DEC_MAX = 90;
    private final float DEC_MIN = -90;

    /**
     * The regular expression describing legal buffer values.
     * Identical to the equivalent in RightAscensionModel,
     * except that these values have an optional sign.
     */
    private final static String legalValuesPattern_ =
    "\\s*(-?\\d{0,2}(:\\d{0,2}(:\\d{0,2}(\\.\\d{0,3})?)?)?)?";

    /**
     * Constructor for the model.
     */
    public DeclinationModel() {
	super(legalValuesPattern_);
    }


    /**
     * Refuse input that is above or below the limits of the
     * textfield. 
     *
     */
    public void insertString(int offset, String str, AttributeSet a)
      throws BadLocationException {

        //stdlog_.debug("String: <"+str+">");
        super.insertString(offset, str, a);
        //stdlog_.debug("buffer: <"+buffer_+">");
        try {
            String buffer_val = buffer_.toString();

            //Do not accept a single :
            if (buffer_val.equals(":")){
                remove(offset, str.length());
            }
            else{
                int len = buffer_val.length();
                if (len > 1){
	            Character x = new Character(buffer_val.charAt(len-2));
                    Character xend = new Character(':');
                    if (x.equals(xend)){
                        //Do not accept a . after a :
                        if(str.equals("."))
                            remove(offset, str.length());

                        //Do not accept a : after a :
                        if(str.equals(":"))
                            remove(offset, str.length());

                        //Check that mins and secs are less than 60
                        if(Integer.parseInt(str) > 5 )
                            remove(offset, str.length());

                        //Do not accept anything after the :
                        //if input is already = -90 or = 90
                        if(len > 3 && len < 6){ 
                            String buf = buffer_val.substring(0,len-2);
                            int bint = Integer.valueOf(buf).intValue();
                            if ((bint == DEC_MAX || bint == DEC_MIN)){
                                int sint = Integer.parseInt(str);
                                remove(offset, str.length());
                            }
                        }
                    }
                    else{
                        Float f = Float.valueOf(buffer_val);
                        if(f.floatValue() > DEC_MAX ) {
                            remove(offset, str.length());
                        }
                        if(f.floatValue() < DEC_MIN ) {
                            remove(offset, str.length());
                        }
                    }
                }
            }

        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }

    }

}
