package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * A Document model for JTextField (and JTextArea) instances that prevents the
 * data becoming longer than a specified size. Normally the limit is imposed
 * because the text needs to fit into a fixed size database column.
 * 
 * <p>
 * @author Tim Canavan
 * @version $Revision: 1.2 $
 */
public class LimitedCharsModel extends PlainDocument {

    private final static String rcsid = "$Id: LimitedCharsModel.java,v 1.2 2004/10/13 12:03:00 tcanavan Exp $";

    /** Maximum number of characters that this is allowed to contain. */
    private int maxLength_ = -1;

    /**
     * @param maxlength
     *            Maximum number of characters that can be typed into the view.
     * 
     * <br>
     * ChangeLog: amchavan, 03-Jan-2002: added.
     */
    public LimitedCharsModel(int maxlength) {
        super();
        maxLength_ = maxlength;
    }

    public LimitedCharsModel() {
        super();
        maxLength_ = ModelsCfg.getCfg().getMaxTextLength();
    }

    /**
     * Inserts characters from the given string into this Document, upto a fixed
     * limit. Normally the limit is imposed because the text needs to fit into a
     * fixed size database column.
     * 
     * @param offset
     *            offset in the text buffer at which to insert string.
     * @param str
     *            string to be inserted.
     * @param attr
     *            text attributes.
     * 
     * @exception BadLocationException
     *                an illegal offset was specified.
     */
    public void insertString(int offset, String str, AttributeSet attr)
            throws BadLocationException {
        int len = getLength();

        if ((maxLength_ >= 0) && (len + str.length() > maxLength_)) {
            // If insertion would make text too long, just return.
            return;
        }

        super.insertString(offset, str, attr);
    }

}

