package org.eso.ohs.core.gui.widgets;

import java.io.File;

import javax.swing.filechooser.FileFilter;

/**
 * A convenience implementation of FileFilter that filters out
 * all files except for those type extensions that it knows about.
 *
 * Extensions are of the type ".foo", which is typically found on
 * Windows and Unix boxes, but not on Macinthosh. Case is ignored.
 *
 * Example - create a new filter that filerts out all files
 * but gif and jpg image files:
 *
 *
 */
public class WildCardExtensionFileFilter extends FileFilter {


    private String extension_;

    private String description_;
  

    /**
     * Creates a file filter. If no filters are added, then all
     * files are accepted.
     */
    public WildCardExtensionFileFilter() {  
    }

    /**
     * Creates a file filter that accepts files with the given extension.
     * Example: new ExtensionFileFilter("jpg");
     */
    public WildCardExtensionFileFilter(String extension) {
	this(extension,null);
    }

    /**
     * Creates a file filter that accepts the given file type.
     * Example: new ExtensionFileFilter("jpg", "JPEG Image Images");
     *
     * Note that the "." before the extension is not needed. If
     * provided, it will be ignored.
     */
    public WildCardExtensionFileFilter(String extension, String description) {
        extension_ = extension;
        description_ = description;
    }


    public String getExtension() {
        return extension_;
    }    

    /**
     * Return true if this file should be shown in the directory pane,
     * false if it shouldn't.
     *
     * Files that begin with "." are ignored.
     *
     * @see #getExtension
     * @see javax.swing.filechooser.FileFilter#accept
     */
    public boolean accept(File f) {
	if(f != null) {
	    if(f.isDirectory()) {
		return true;
	    }       	
            String filename = f.getName();
	    int i = filename.lastIndexOf(extension_);
	    if(i != -1)
	        return true;	 	    	   
	}
	return false;
    }

    public String getDescription() {
        return "(*" + extension_ + ")";
    }        
}
