package org.eso.ohs.dfs;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * This implemetns the static creation pattern as per Juscha Block
 * effective Java. 
 * It is also desciribed on Java world at the following address.
 * http://www.javaworld.com/javaworld/jw-06-2001/j1-01-sintes2.html
 * 
 * <p>
 * $Id: ObjectFactory.java,v 1.10 2005/01/28 09:47:52 tcanavan Exp $<br>
 * 
 * @author Bob Kemp
 * @author Tim Canavan 
 * Removed singleton changed to simple static methods.
 */

public class ObjectFactory {

    

    /**
     * This is the factory method which creates a Business Object
     * 
     * @param objType
     *        The type of a given object Target.class etc..
     * @return BusinessObject A fully instanciated business object
     */
    
    
    

    public static BusinessObject create(Class objType) {
        if (BusinessObject.class.isAssignableFrom(objType)) {
            try {
                return (BusinessObject) objType.newInstance();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InstantiationException e) {
                e.printStackTrace();
            }

            throw new IllegalArgumentException("Cannot create: "
                    + objType.getName());
        }

        throw new IllegalArgumentException("Not a BusinessObject: "
                + objType.getName());
    }

    /**
     * This class creates a business object ad all its associated business
     * objects via Recursion and Java Reflection
     * 
     * @param objType
     *            Class name for a business object
     * @return Reference to a BusinessObject
     */

    public static BusinessObject createRecursive(Class objType) {
        BusinessObject bo = create(objType);
        Method[] meths = objType.getMethods();

        for (int j = 0; j < meths.length; j++) {
            Class retType = meths[j].getReturnType();

            if (ObservingRun.class.isAssignableFrom(retType)) {
                continue;
            }

            if (BusinessObject.class.isAssignableFrom(retType)) {
                String propName = meths[j].getName().substring(3);
                String setterName = "set" + propName;
                for (int k = 0; k < meths.length; k++) {
                    if (setterName.equals(meths[k].getName())) {
                        BusinessObject subBO = createRecursive(retType);

                        try {
                            meths[k].invoke(bo, new Object[] { subBO });
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                            throw new IllegalArgumentException(
                                    "Constructor inaccessible: "
                                            + retType.getName());
                        } catch (InvocationTargetException e) {
                            e.getTargetException().printStackTrace();
                            throw new IllegalArgumentException(
                                    "Exception in constructor: "
                                            + retType.getName()
                                            + ": "
                                            + e.getTargetException()
                                                    .getMessage());
                        }
                    }
                }
            }
        }
        return bo;
    }
}

