package org.eso.ohs.instruments;

/**
 * This constains all the configuration information about a
 * constraint set. 
 * @author tcanavan
 *
 */
public class InstrumentConstraintConfig {

    private String label;

    private String dbColumn;

    private String pafKeyword = "None";

    private InstrumentConstraintMetaData metaData;

    private String name;

    private String fkColumn;

    private String fkTable;
    
    private String toolTip;

    private boolean property = false;
    
    
    // Value must be entetred.
    
    private boolean compulsory = false;


    private String superType = "text";

    public InstrumentConstraintConfig() {
        metaData = new InstrumentConstraintMetaData();
    }

    /**
     * Return the database column associated with a constraint
     * @return The column associated with a database constraint
     */
    public String getDbColumn() {
        return dbColumn;
    }

    /**
     * Set the database column associated with a constraint
     * @param string
     */
    public void setDbColumn(String string) {
        dbColumn = string;
    }

    /**
     * Get the label associated with a database constraint.
     * @return The label text for GUI display.
     */
    public String getLabel() {
        return label;
    }

    /**
     * Set the label associated with a database constraint.
     * @param The label text for GUI display.
     */

    public void setLabel(String string) {
        label = string;
    }

    /**
     * Return the validation metainformation for the a constraint
     * @return validation metainformation for the a constraint
     */
    public InstrumentConstraintMetaData getMetaData() {
        return metaData;
    }

    /**
     * Set the validation metainformation for the a constraint
     * 
     * @param data
     */
    public void setMetaData(InstrumentConstraintMetaData data) {
        metaData = data;
    }

    /**
     * Get the paf keyword where the constraint is derived from a 
     * a paf file keyword. 
     * @return The Paf keyword 
     */

    public String getPafKeyword() {
        return pafKeyword;
    }

    /**
     * Set the paf keyword where the constraint is derived from
     * a param file
     * @param The paf keyword 
     */

    public void setPafKeyword(String string) {
        pafKeyword = string;
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("PAF KEYWORD" + getPafKeyword());
        buf.append(" LABEL " + getLabel());
        buf.append("Db Column " + getDbColumn() + "\n");
        buf.append("Meta Info" + getMetaData().toString() + "\n");
        return buf.toString();
    }

    /**
     * The base type attribute of the readme
     * @return The base type of an attribute a string,float etc...
     */

    public String getBaseType() {
        return metaData.getBaseType();
    }

    /**
     * The name of the constraint
     * @return Name of the constraint
     */
    public String getName() {
        return name;
    }

    /**
     * Set the name of the constraint
     * @param name of the constraint
     */
    public void setName(String string) {
        name = string;
    }

    /**
     * Set the foreigh key column if the constraint
     * is stored in another table.
     * @return Foreign key table.
     */
    public String getFkColumn() {
        return fkColumn;
    }

    /**
     * Get the foreigh key table if the constraint
     * is stored in another table.
     * @return The foreign key table.
     */
    public String getFkTable() {
        return fkTable;
    }

    /**
     * Set the foreign key table if the constraint
     * is stored in another table.
     * @param Set the foreign key column. 
     */
    public void setFkColumn(String string) {
        fkColumn = string;
    }

    /**
     * Set the foreign key table if the constraint
     * is stored in another table.
     * @param Set the foreign key table. 
     */

    public void setFkTable(String string) {
        fkTable = string;
    }

    public boolean hasFCConstraint() {
        if ((fkColumn != null && fkColumn.length() > 0)
            && (fkTable != null && fkTable.length() > 0))
            return true;
        return false;
    }

    /**
     * Get the suprttype
     * @return Get the supertype.
     */
    public String getSuperType() {
        int maxLength = metaData.getMaxLength();
        
        if(superType.equals("emailAddressType")) {
            return "emailAddressType";
        }
        if (superType.equals("ThreeStateBooleanType")) {
            return "ThreeStateBooleanType";
        }
        if (superType.equals("FourStateBooleanType")) {
            return "FourStateBooleanType";
        }
        if (getBaseType().equals("integer")) {
            return "integer";
        }
        if (getBaseType().equals("float")) {
            return "float";
        }
        if(superType.equals("multilist")) {
            return "multilist";            
        }        
        if (getBaseType().equals("string") && getMetaData().getAllowedValues().length > 0) {
            return "keywordlist";
        }
        if (getBaseType().equals("string") && maxLength > 120) {
            return "largetext";
        }
        if (getBaseType().equals("string") && maxLength <= 120 && maxLength > 0) {
            return "normaltext";
        }
        return superType;
    }

    /**
     * @param string
     */
    public void setSuperType(String string) {
        superType = string;
    }

    /**
     * Is Propery   
     * @return Is property
     */
    public boolean isProperty() {
        return property;
    }

    /**
     * 
     * @param b
     */
    public void setProperty(boolean b) {
        property = b;
    }

    /**
     * Get a compulsory attribute.
     * @return Is a compulsory attribute.
     */
    public boolean isCompulsory() {
        return compulsory;
    }

    /**
     * Set isCompulsory.
     * @param b
     */
    public void setCompulsory(boolean b) {
        compulsory = b;
    }

    /**
     * @return tooltip text
     */
    public String getToolTip() {
        if(toolTip==null)
            return "";
        return toolTip;
    }

    /**
     * Set tooltip text
     * @param string
     */
    public void setToolTip(String string) {
        toolTip = string;
    }

}
