package org.eso.ohs.persistence.dbase.phase2;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.persistence.DirectoryNode;

/**
 * The storage manager class for files stored in the database.
 * A concrete subclass of StorageManager.
 * <p>
 * NOTE: we could not instantiate the Sybase driver in the normal way, using
 * Class.forName().  Instead it is necessary to follow the special instruction
 * in the Sybase JDBC driver installation instructions.  This is to update the
 * Java system properties.
 * <p>
 * @version $Revision: 1.1 $
 * @author Bob Kemp
 */
public class P2PPDbaseMgr extends DbaseMgr {

    public static final String cvsID_ =
        "$Id: P2PPDbaseMgr.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $";

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(OTDbaseMgr.class);


    /**
     * Default constructor
     */
    public P2PPDbaseMgr() {
        super();
    }


    /**
     * Builds a list of Summary objects for a given directory.
     * These Summary objects contain the unique Id of the object
     * and the Class which contains details about the type of Object
     * that is being referenced be it an Observation Description,Observation
     * Block etc..
     *
     * @param node Directory from which you can create a list of objects
     * @exception ObjectIOException if there was an I/O error listing an object
     * @return Array of ObjectFileInfo
     */
    public Summary[] listObjects(DirectoryNode node)
                                throws ObjectIOException {
        Vector vec = new Vector();
        Connection con = null;
        Statement stmt = null;

        try {
            con = acquireConnection();
            stmt = con.createStatement();
            stdlog_.debug( "URL: " + con.getMetaData().getURL() );

            addObjectFileEntry(stmt, vec,
                ObservationBlock.class,
                "obs_blocks,target_packages,obs_descriptions",
                "obs_blocks.user_id=" + getUserId(),
                "obs_blocks.tp_id = target_packages.tp_id"
                + " AND obs_blocks.ob_id = obs_descriptions.od_id"
                + " AND type = '" + CalibrationBlock.OBSBLK_TYPE + "'"
            );

            addObjectFileEntry(stmt, vec,
                CalibrationBlock.class,
                "obs_blocks,obs_descriptions",
                "obs_blocks.user_id=" + getUserId(),
                "obs_blocks.ob_id = obs_descriptions.od_id"
                + " AND type = '" + CalibrationBlock.CALBLK_TYPE + "'"
            );

            addObjectFileEntry(stmt, vec,
                Target.class,
                "target_packages",
                "user_id=" + getUserId(),
                ""
            );

            addObjectFileEntry(stmt, vec,
                ObservationDescription.class,
                "obs_descriptions",
                "user_id=" + getUserId(),
                ""
            );

            addObjectFileEntry(stmt, vec,
                ConstraintSet.class,
                "constraint_sets",
                "user_id=" + getUserId(),
                ""
            );

            stmt.close();
            stmt = null;

        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectIOException(e);

        } finally {
            try {
                if (stmt != null) {
                    stmt.close();
                }
            } catch (SQLException e) {
                e.printStackTrace();
            } finally {
                if (con != null) {
                    releaseConnection(con);
                }
            }
        }

        Summary[] sums = new Summary[vec.size()];
        vec.copyInto(sums);
        return sums;
    }
}

