package org.eso.ohs.phase2.apps.ot.dbb;

// $Id: OTDbbView.java,v 1.20 2005/11/30 16:48:35 ddorigo Exp $

import java.awt.Container;

import javax.swing.CellEditor;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbTableSortEvent;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbQueryStarted;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelectionEvent;
import org.eso.ohs.persistence.ObjectSelectionListener;
import org.eso.ohs.persistence.ObjectSelector;

/**  
 *
 * @author Giorgio Passera ESO 15-Sep-2000
 */

public class OTDbbView
    extends DbbView
    implements ObjectContainer, ObjectSelector {
 
    private static final long serialVersionUID = 1L;

        static public final String OB_ID = "OB ID";

        static public final String TYPE = "Type";

        public static final String STATUS = "Status";

        public static final String QUEUE_ID = "QueueID";
 

        public static final String ORANG_RANK = "Rank";


        public static final String RUN_ID = "Run Id";

        public static final String ORANG_RANK_HIDE = "RankHide"; 

        private static Logger stdlog_ =
            Logger.getLogger(OTDbbView.class);                    
    
    private EventListenerList listenerList_ = new EventListenerList();

    //----------------------------------------
    // Constructor
    //----------------------------------------

    /**
     * Build a database browser view.
     *
     * @param sqlEngine Reference to the SQL engine which will do all
     *                  the real work.
     */
    public OTDbbView(
        DbbSqlEngine sqlEngine,
        boolean clearButton,
        boolean queryButton,
        boolean sortButton) {

        /* Call the DbbView constructor. */
        super(sqlEngine);
        
        //stdlog_.debug("Afer constructing Engine");
        if (!sortButton)
            hideSortButton();
        hideCountButton();
        hideStopButton();
        if (!clearButton)
            hideResetButton();
        if (!queryButton)
            hideQueryButton();     
    }
    
    public void createDefaultListListener() {
        addListSelectionListener(new ListListener(summaryTable_.getTable()));
    }

    public OTDbbView(DbbSqlEngine sqlEngine) {
        super(sqlEngine);
    }

    public void sqlStartAction() {
        super.sqlStartAction(new DbbQueryStarted(this));
    }

    public void tableSortAction(DbbTableSortEvent event) {
        super.tableSortAction(event);       
    }

    /**
     *Removes a set of rows
     *@param rows specifies a set of row's indeces to be removed
     */
    public void removeRows(int[] rows) {
        summaryTable_.removeRows(rows);
    }

    /**
    *MovesUp a set of rows
    *@param rows specifies a set of row's indeces to be moved up
    */
    public void moveUpRows(int[] rows) {
        summaryTable_.moveUpRows(rows);
    }

    /**
     *MovesUp a set of rows
     *@param rows specifies a set of row's indeces to be moved up
     */
    public void moveToTop(int[] rows) {
        summaryTable_.moveToTop(rows);
    }

  
    public void setValueAt(Object object, int row, String column) {
        int colIndex = summaryTable_.getColIndex(column);
        summaryTable_.getModel().setValueAt(object, row, colIndex);
    }

    /**
     *MovesUp a set of rows
     *@param rows specifies a set of row's indeces to be moved up
     */
    public void moveDownRows(int[] rows) {
        summaryTable_.moveDownRows(rows);
    }

    public Class getClassForId(long id) throws NullPointerException {
        int rowCount = summaryTable_.getRows();
        for (int i = 0; i < rowCount; i++) {
            Integer rowid = (Integer) getValueAt(i,OB_ID);
            // stdlog_.debug("Row Id in table " + rowid.longValue());
            String type = (String) getValueAt(i,TYPE);
            if (rowid.longValue() == id) {
                if (type.equalsIgnoreCase("O"))
                    return ObservationBlock.class;
                if (type.equalsIgnoreCase("C"))
                    return CalibrationBlock.class;
            }
        }
        return null;
    }

    public void selectAllObjects() {
        ListSelectionModel lsm = summaryTable_.getTable().getSelectionModel();
        CellEditor ed = summaryTable_.getTable().getCellEditor();
        if (ed != null) {
            ed.stopCellEditing();
        }
        lsm.addSelectionInterval(0, summaryTable_.getModel().getRowCount() - 1);
    }

    public Container getTopLevelAncestor() {
        return ((Container) summaryTable_.getTable().getTopLevelAncestor());
    }

    public Long[] getAllSelected() {
        int[] selectedRows = getSelectedRows();

        Long[] selectedOBsId = new Long[selectedRows.length];
        for (int i = 0; i < selectedRows.length; i++) {
            Integer value =
               (Integer) getValueAt(selectedRows[i],OB_ID);
            String type = (String) getValueAt(selectedRows[i],TYPE);
            if (type.equals("O")) {
                selectedOBsId[i] =
                    new Long(
                        Config.getCfg().tableToUniqueId(
                            value.longValue(),
                            ObservationBlock.class));
            } else {
                selectedOBsId[i] =
                    new Long(
                        Config.getCfg().tableToUniqueId(
                            value.longValue(),
                            CalibrationBlock.class));
            }
        }
        return selectedOBsId;
    }

    public Long getSelected() {
        Class objTypeSelected;
        int row = getSelectedRow();
        
        
        if (row == -1)
            return new Long(row);
        Integer id =(Integer) getValueAt(row,OB_ID);
        String type = (String) getValueAt(row,TYPE);
        if (type.equalsIgnoreCase(CalibrationBlock.OBSBLK_TYPE))
            objTypeSelected = ObservationBlock.class;
        else
            objTypeSelected = CalibrationBlock.class;

        Long dbid =
            new Long(
                Config.getCfg().tableToUniqueId(
                    id.longValue(),
                    objTypeSelected));

        return dbid;
    }

    public Media getMedia() {
        return Media.DBASE;
    }

    public Long[] getAll() {
        int selectedRows = getTable().getRows();
        Long[] selectedOBsId = new Long[selectedRows];
        for (int i = 0; i < selectedRows; i++) {
            Integer value = (Integer) getValueAt(i,OB_ID);
            String type = (String) getValueAt(i,TYPE);
            if (type.equals("O")) {
                selectedOBsId[i] =
                    new Long(
                        Config.getCfg().tableToUniqueId(
                            value.longValue(),
                            ObservationBlock.class));
            } else {
                selectedOBsId[i] =
                    new Long(
                        Config.getCfg().tableToUniqueId(
                            value.longValue(),
                            CalibrationBlock.class));
            }
        }
        return selectedOBsId;
    }
    /**
     * @see org.eso.ohs.persistence.ObjectSelector#getDisplayedObject()
     */
    public BusinessObject getDisplayedObject() {
        Long selected = getSelected();
        if(selected.longValue()== -1)
             return null;
        ObjectManager objMgr = ObjectManager.getObjectManager();
        Class c = Config.getCfg().getClassFromId(selected.longValue());
        CalibrationBlock cb;
        try {
            cb =
                (CalibrationBlock) objMgr.getBusObj(
                    Media.DBASE,
                    selected.longValue(),
                    c);
        } catch (ObjectNotFoundException e) {
            ErrorMessages.announceNoObject(summaryTable_.getTable(),e);
            return null;
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(summaryTable_.getTable(),e);
            return null;
        }
        return cb;
    }
    
    public Class getDisplayedObjectClass() {
        Long selected = getSelected();
        if(selected.longValue()== -1)
             return null;
            
        Class c = Config.getCfg().getClassFromId(selected.longValue());
        return c;        
    }


   

 
    
    protected void fireObjectSelectionEvent(
        Media device,
        long id,
        Class objType,
        boolean forceShow,
        boolean forceToFront,
        int rowCount) {

        Object[] listeners = listenerList_.getListenerList();
        ObjectSelectionListener lnr = null;
        Class visibleObjType = getDisplayedObjectClass();
        ObjectSelectionEvent selnEvt =
            new ObjectSelectionEvent(
                this,
                device,
                id,
                visibleObjType,
                forceShow,
                forceToFront,
                rowCount);

        for (int i = 0; i < listeners.length; i += 2) {
            lnr = (ObjectSelectionListener) listeners[i + 1];
            lnr.objectSelected(selnEvt);
        }
    }

    /**
     * Adds a new listener to be notified of selection changes.
     * 
     * @param lnr the listener
     */
    public void addObjectSelectionListener(ObjectSelectionListener lnr) {
        listenerList_.add(ObjectSelectionListener.class, lnr);
    }

    /**
     * Removes a new listener, which will no longer be notified of
     * selection changes.
     * 
     * @param lnr the listener
     */
    public void removeObjectSelectionListener(ObjectSelectionListener lnr) {
        listenerList_.remove(ObjectSelectionListener.class, lnr);
    }

    public class ListListener implements ListSelectionListener {

        JTable summaryTable_;

        public ListListener() {
            summaryTable_ = null;
        }

        public ListListener(JTable summaryTable) {
            summaryTable_ = summaryTable;
        }

        public void valueChanged(ListSelectionEvent ev) {
            stdlog_.debug("List Selection Event "  + ev);
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            int selectedRow = lsm.getMinSelectionIndex();
            Long idObj = getSelected();

            long id = ObjectSelectionEvent.NO_OBJECT;

            if (!lsm.isSelectionEmpty()) {
                if (selectedRow >= 0
                    && selectedRow < summaryTable_.getModel().getRowCount()) {
                    id = idObj.longValue();
                }
                fireObjectSelectionEvent(
                    Media.DBASE,
                    id,
                    Config.getCfg().getClassFromId(id),
                    false,
                    true,
                    summaryTable_.getModel().getRowCount());
            }
            if (id != ObjectSelectionEvent.NO_OBJECT)
                fireObjectSelectionEvent(
                    Media.DBASE,
                    id,
                    Config.getCfg().getClassFromId(id),
                    false,
                    true,
                    summaryTable_.getModel().getRowCount());
            else
                fireObjectSelectionEvent(
                    Media.DBASE,
                    id,
                    ObservationBlock.class,
                    false,
                    true,
                    summaryTable_.getModel().getRowCount());
        }
    }

}