package org.eso.ohs.phase2.apps.ot.gui;

import java.io.File;
import java.io.IOException;
import java.util.Properties;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.docview.datatrans.AdaptorFactory;
import org.eso.ohs.core.gui.models.ModelsCfg;
import org.eso.ohs.core.gui.widgets.LoginDialog;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ExtFileList;
import org.eso.ohs.core.utilities.NotASCIIException;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjConfig;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.instruments.InstrumentInitialiser;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.AuthenticatingStorageMgr;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSAdaptorFactory;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ProtectingStorageMgr;
import org.eso.ohs.persistence.StorageManager;
import org.eso.ohs.persistence.dbase.phase1.OpcDbaseMgr;
import org.eso.ohs.persistence.dbase.phase2.DBInstrumentListFactory;
import org.eso.ohs.persistence.dbase.phase2.DbaseMgr;
import org.eso.ohs.persistence.dbase.phase2.OTDbaseMgr;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.apps.ot.MyIcdVcsCfg;
import org.eso.ohs.phase2.apps.ot.MyPersistCfg;
import org.eso.ohs.phase2.apps.ot.OTVersion;
import org.eso.ohs.phase2.apps.ot.Persistence;
import org.eso.ohs.phase2.icdVcs.CCSConnect;
import org.eso.ohs.phase2.icdVcs.IcdVcsCfg;

/**
 * A simple class to get the Java OT started.
 * 
 * <p>
 * <b>README README README README README README README README README </b>
 * 
 * In the places where it is clear to me that you will (eventually) need to
 * replace the code (eg where I have put a hardwired value that works for now),
 * I have written "XXX" in a comment. This means "needs attention -- later"
 * 
 * <p>
 * Just to get you started, I have used various components from JP2PP. You
 * should try to get rid of these dependencies as soon as possible. However, it
 * may be that some of this JP2PP code can be moved into shared packages.
 * 
 * <b>README README README README README README README README README </b>
 * <p>
 * 
 * @version $Id: OT.java,v 1.33 2005/11/25 13:07:57 pnunes Exp $<br>
 * @author Bob Kemp (put your name here :-) <br>
 *         ChangeLog: cmavros 11-03-2002 1700/OT <br>
 *         ChangeLog: cmavros 08May02 [1426/1501/1631/OT] Add call to
 *         offsetFontSize <br>
 *         ChangeLog: cmavros 05JUL02 [1836/OT] Change config output from debug
 *         to info
 */
public class OT extends OHSApplication {

    OT(String[] args) {

        // Initialize this application
        OHSApplication ot = getApplication();
        if ("true".equals(System.getProperty("fcviewer"))) {
            ot.init("Finding Chart Viewer", "FCV", OTVersion.getVersion());
        } else {
            ot.init("Observing Tool", "OT", OTVersion.getVersion());
        }
        StorageManager opcDbaseMgr;
        StorageManager obrepDbaseMgr;
        String homeDir = System.getProperty("user.home");        
        System.setProperty("java.util.prefs.systemRoot",homeDir);
        try {
            setLookAndFeel();
            setupConnectionPool();
            AppConfig appcfg = new AppConfig();
            AppConfig.setAppConfig(appcfg);
            AppConfig.getAppConfig().setStandardOHSKeys("config");
            OHSApplication.checkDbFileExists();
            readCfgs();
            setupSubsystems();
            //AppConfig.getAppConfig().setValue(OHSKeys.DefaultPeriodKey, 70);
            stdlog_.info("CONFIG:\n" + AppConfig.getAppConfig().toString());
            initObjInfo();
            Config cfg = Config.getCfg();
            opcDbaseMgr = createStorageManager(Media.PHASE1_DB);
            obrepDbaseMgr = createStorageManager(Media.PHASE2_DB);
            //Set font sizes
            offsetFontSize(appcfg);
            setDefaultGUIResources();
            checkInstrumentDirectories(appcfg);

            String ipCacheDir = AppConfig.getAppConfig().getIPVersionIPCache();
            if (AppConfig.getAppConfig().isIPVersioningSingleIPMode()
                    && AppConfig
                            .getAppConfig()
                            .getIPVersioningAutodownloadIPEnabled()) {
                JOptionPane
                        .showMessageDialog(
                                null,
                                new Object[]
                                {
                                        "Single IP Mode may not be enabled along with automatic IP download.",
                                        " ",
                                        "Automatic IP Download Keyword: ",
                                        "    "
                                                + OHSKeys.IPVersioningAutodownloadIP,
                                        "Single IP Mode Keyword: ",
                                        "    "
                                                + OHSKeys.IPVersioningSingleIPModeEnabled,
                                        " ", "Cannot continue!" },
                                "Configuration Error",
                                JOptionPane.ERROR_MESSAGE);
                System.exit(0);
            } else {
                if (AppConfig
                        .getAppConfig()
                        .getIPVersioningAutodownloadIPEnabled()) {
                    if (ipCacheDir == null || ipCacheDir.length() == 0) {
                        JOptionPane.showMessageDialog(null, new Object[]
                        { "The Auto IP download option is enabled",
                                "but no IP cache folder is configured.", " ",
                                "Automatic IP Download Keyword: ",
                                "    " + OHSKeys.IPVersioningAutodownloadIP,
                                "IP Cache Directory Keyword: ",
                                "    " + OHSKeys.IPVersioningIPCache, " ",
                                "Cannot continue!" }, "Configuration Error",
                                JOptionPane.ERROR_MESSAGE);
                        System.exit(0);
                    }
                }
            }
            int loginStatus = login(null, opcDbaseMgr);

            //end login with control
            if (loginStatus < 0) {
                System.exit(0);
            }
            boolean ipMode = AppConfig
                    .getAppConfig()
                    .isIPVersioningSingleIPMode();
            String instDir = AppConfig.getAppConfig().getInstrumDir();
            if (ipCacheDir != null && ipCacheDir.length() > 0) {
                checkDirectory(new File(ipCacheDir), true, true);
                /*
                 * Load the DBInstrumentListFactory() if set in the
                 * configuration.
                 */
                if (!AppConfig.getAppConfig().isIPVersioningSingleIPMode()
                        && AppConfig
                                .getAppConfig()
                                .getIPVersioningAutodownloadIPEnabled()) {
                    /*
                     * Code here assumes that the configuration is correct -
                     * i.e. that the ipCacheDir exists in the configuration.
                     * This is tested above.
                     */
                    InstrumentList
                            .setInstrumentListFactory(new DBInstrumentListFactory(
                                    ipMode, instDir, ipCacheDir, AppConfig
                                            .getAppConfig()
                                            .getDbUserName(), AppConfig
                                            .getAppConfig()
                                            .getDbPassword()));
                }
            }
            InstrumentInitialiser instrInit = new InstrumentInitialiser(ipMode,
                    instDir, ipCacheDir);
            instrInit.start();
            initialiseStorage();
            if (!instrInit.checkInstrumentsLoaded()) {
                JOptionPane.showMessageDialog(null,
                        "There are no Instrument Packages in folder\n"
                                + instDir, "No Instrument Packages found",
                        JOptionPane.ERROR_MESSAGE);

                System.exit(0);
            }
            if (instrInit.checkInstrumentsVersion()) {
                stdlog_.debug("All instrument packages have correct version");
            }

            
            displayInstrumentErrorInfo();
            initialiseTargetObjectClass();
            String optDirStr = appcfg.getOptionDir();
            System.out.println("OPT DIR: <" + optDirStr + ">");
            if (optDirStr != null && optDirStr.length() > 0) {
                File optDir = new File(optDirStr);
                if (!optDir.exists()) {
                    File createOptDir = new File(optDir.getAbsolutePath());
                    if (optDir.mkdir()) {
                        JOptionPane.showMessageDialog(null,
                                "The system just created the options directory "
                                        + "\nin " + optDir, "Warning message",
                                JOptionPane.WARNING_MESSAGE);
                    } else {
                        JOptionPane.showMessageDialog(null,
                                "The system could not create the options directory "
                                        + "\nin " + optDir, "Warning message",
                                JOptionPane.WARNING_MESSAGE);
                        System.exit(0);
                    }
                } else if (!optDir.canWrite()) {
                    JOptionPane.showMessageDialog(null,
                            "The system cannot write to the options directory "
                                    + "\nin " + optDir, "Warning message",
                            JOptionPane.WARNING_MESSAGE);
                    System.exit(0);
                }
            }
        } catch (Exception e) {
            stdlog_.error("", e);
            ;
            System.exit(1);
        }
        QueueManager.createTmpTables();

        // BOB connection & MaskListener
        // A M Chavan, 03-Feb-01
        AppConfig appcfg = AppConfig.getAppConfig();

        if (appcfg.getCCSEnabled() && appcfg.getCCSServer() != null) {

            // BOB connection, no MaskListener
            // A M Chavan, 03-Feb-01
            boolean t = false;
            CCSConnect ccsConnect = new CCSConnect();
            ccsConnect.startCcs("org.eso.ohs.phase2.apps.ot.icdVcs.OTBOB");
            appcfg.setCcsActive(t);
        } else {
            stdlog_.info("Access to BOB is not configured.");
        }
        //OTViewManager manager = new OTViewManager();
        ///OTViewManager manager = OTViewManager.getOTViewMgr();
        if ("true".equals(System.getProperty("fcviewer"))) {
            FindingChartBrowser browser = new FindingChartBrowser(
                    "Finding Chart Viewer V." + OHSApplication.getVersion()
                            + " User=" + Config.getCfg().getUserName()
                            + " Server="
                            + AppConfig.getAppConfig().getDbServer());
        } else {
            OTMainView otMainView = new OTMainView();
        }
//        try {
//            Phase1SelectStmt.definePhase2TableDb("obrep2","dbo");
//            UnpackedStorableObject[] obList = OpcDbaseMgr.getInstance().read(null,
//                    "run_id > 75047000", ObTargetConstraintInfo.class);
//   stdlog_.debug(" _____________________- "+obList.length);
//            
//        } catch (ObjectNotFoundException e1) {
//            // TODO Auto-generated catch block
//            e1.printStackTrace();
//        } catch (ObjectIOException e1) {
//            // TODO Auto-generated catch block
//            e1.printStackTrace();
//        }
        
        //manager.createView();

    }

    /**
     * Set up required, app specific, class implementations.
     */
    protected void setupSubsystems() throws IOException {
        Config.setCfg(new MyPersistCfg());
        IcdVcsCfg.setCfg(new MyIcdVcsCfg());
        ModelsCfg.getCfg().setMaxTextLength(
                AppConfig.getAppConfig().getPersistenceMaxTextLength());
        ModelsCfg.getCfg().setMaxNameLength(
                AppConfig.getAppConfig().getPersistenceMaxNameLength());
        AdaptorFactory.setInstance(new OHSAdaptorFactory());
    }

    /**
     * Read required config files into AppConfig.
     */
    protected void readCfgs() throws IOException {
        AppConfig ac = AppConfig.getAppConfig();
        ExtFileList extList = new ExtFileList(ac.getConfigDir(), ".cf");
        String dbConfig = "ot-db.cf";
        if ("true".equals(System.getProperty("fcviewer"))) {
        	dbConfig="fcv-db.cf";
        }
        if (extList.exists(dbConfig)) {
            File dbFile = extList.getFileHandle(dbConfig);
            if(!dbFile.canRead()) {
                JOptionPane.showMessageDialog(
                        null,
                        "Cannot read config file " + dbConfig,
                        "File Read Permission Error : " + dbConfig,
                        JOptionPane.ERROR_MESSAGE);
                    System.exit(0);

            } 
            //ac.setupDbDirectConfig(OHSKeys.DBDirectSubConfig, dbFile);
            // need to put db config file in root dir (not in a sub config) or
            // it can not
            // be overridden from the users config file.
            ac.setupDbDirectConfig("", dbFile);
        } else {
            JOptionPane.showMessageDialog(
                    null,
                    "Database config file missing : " + dbConfig,
                    "Database config file missing : " + dbConfig,
                    JOptionPane.ERROR_MESSAGE);
                System.exit(0);
        }
        String currDir = System.getProperty("user.dir");
        File siteCf = extList.getFileHandle("site.cf");
        if (siteCf != null) {
            if (siteCf.exists() && siteCf.canRead()) {
                try {
                    ac.addParamfile(siteCf.getAbsolutePath());
                } catch (NotASCIIException e) {
                    // cmavros 11-03-2002 1700/OT
                    int rtn = JOptionPane
                            .showConfirmDialog(
                                    null,
                                    "Syntax Error while reading:\n"
                                            + siteCf
                                            + "\non the following lines:"
                                            + "\n\n"
                                            + e.getMessage()
                                            + "\nIgnoring syntax errors can cause undesirable results."
                                            + "\n\nPlease quit and correct the syntax errors before continuing."
                                            + "\n\nDo you want to quit?",
                                    "Syntax Error", JOptionPane.YES_NO_OPTION);
                    if (rtn == JOptionPane.YES_OPTION) {
                        System.exit(0);
                    }
                }
            }
        }
        //if (!ac.getStartAppServer()) {
        File userCf = new File(ac.getHomeDir() + File.separator + ".ot.cf");
        if (userCf.exists() && userCf.canRead()) {
            // the User CF file over writes settings in the site.cf
            // file - these settings are stored under a
            // UserOverrideKey subconfig
            // and used to over write settings when required.
            // Examine prependBase() for details on the effects of
            // the oser-ridden parameters.
            try {
                ac.addParamfile(userCf.getAbsolutePath(),
                        OHSKeys.UserOverrideKey);
                ac.processOverrides();
            } catch (NotASCIIException e) {
                throw new IOException(e.getMessage());
            }
        }

        // Here we supersede the contents of the CCS config file, if
        // any, with the value of the "ccs.wanted" property.
        // Under Unix, that can be selected using the "-noccs"
        // command line option.
        //
        // A M Chavan, 17-Jan-2000

        if (ac.getCCSEnabled()
                && "false".equals(System.getProperty("ccs.wanted"))) {
            ac.setValue(OHSKeys.CcsConnEnabledKey, false);
        }
    }

    /**
     * Initialize the required business object definitions.
     */
    protected void initObjInfo() {
        AppConfig appCfg = AppConfig.getAppConfig();
        appCfg.putObjCfg(new ObjConfig(ConstraintSet.class, new String[]
        {
        // Displayed column names
                "Id", "Name", "Airmass", "Sky Transparency",
                "Lunar Illumination", "Seeing", "Moon Distance" }, new String[]
        { // Names of displayed properties
                "Id", "Name", "Airmass", "SkyTransparency",
                        "FractionalLunarIllumination", "Seeing",
                        "MoonAngularDistance" }, new String[]
                { // Editable property names
                "Name", "Airmass", "FractionalLunarIllumination", "Seeing",
                        "MoonAngularDistance" }));

        appCfg.putObjCfg(new ObjConfig(Target.class, new String[]
        {
        // Displayed column names
                "Id", "Name", "Ra", "Dec", "Equinox", "Epoch" }, new String[]
        { // Names of displayed properties
                "Id", "Name", "RA", "Declination", "Equinox", "Epoch" },
                new String[]
                { // Editable property names
                "Name", "RA", "Declination", "Equinox", "Epoch" }));

        appCfg
                .putObjCfg(new ObjConfig(ObservationBlock.class, new String[]
                {
                // Displayed column names
                        "Id", "Name", "DbaseId", "Status", "Target", "OD",
                        "CS", "Acquisition" }, new String[]
                { // Names of displayed properties
                        "Id", "Name", "DbaseId", "Status", "Target", "Od",
                                "ConstraintSet", "TemplateSignature" },
                        new String[] {}));

        appCfg.putObjCfg(new ObjConfig(CalibrationBlock.class, new String[]
        {
        // Displayed column names
                "Id", "Name", "DbaseId", "Status", "OD" }, new String[]
        { // Names of displayed properties
                "Id", "Name", "DbaseId", "Status", "Od" }, new String[] {}));

        appCfg.putObjCfg(new ObjConfig(ObservationDescription.class,
                new String[]
                {
                // Displayed column names
                        "Id", "Name", "Instrument" }, new String[]
                { // Names of displayed properties
                "Id", "Name", "Instrument" }, new String[]
                { // Editable property names
                "Name" }));

        appCfg.putObjCfg(new ObjConfig(ObservingRun.class, new String[]
        {
        // Displayed column names
                // Period is required by MainWindow.Phase1Information
                "Id", "Period", "WizardTitlePanel" }, new String[]
        { // Names of displayed properties
                // Period is required by MainWindow.Phase1Information
                        "Id", "Period", "WizardTitlePanel" }, new String[] { // Editable
                                                                  // property
                                                                  // names
                }));

        appCfg.putObjCfg(new ObjConfig(Queue.class, new String[]
        {
        // Displayed column names
                // Period is required by MainWindow.Phase1Information
                "Queue Id", "Name", "Size", "Telescope" }, new String[]
        { // Names of displayed properties
                // Period is required by MainWindow.Phase1Information
                        "QueueId", "Name", "Size", "Telescope" }, new String[] { // Editable
                                                                                 // property
                                                                                 // names
                }));

    }

    /**
     * Initialise the Objectmanager and StorageManagers. These classes are used
     * to read/write to local storage and/or dbase.
     */
    private static void initialiseStorage() throws ObjectIOException {
        // Should use ObjectManager but we're (temporarily)
        //  using code from p2pp package, which assumes that
        //  ObjectManager is configured.
        Persistence.startup();
        Persistence objMgr = Persistence.getInstance();
        objMgr.setStorageManager(Media.PHASE1_DB, new OpcDbaseMgr());
        objMgr.setStorageManager(Media.PHASE2_DB, new OTDbaseMgr());

    }

    /**
     * Setup the connection pool properties prior to its first use. Note: the OT
     * requires a single connection that is never closed. This is because of its
     * use of temporary #tables in Sybase.
     */

    private void setupConnectionPool() {

        Properties p = new Properties();
        p.put(ConnectionPool.PoolSize, "1");
        p.put(ConnectionPool.Timeout, "0");
        p.put(ConnectionPool.ReopenAllowed, "F");

        if (!ConnectionPool.setInitProperties(p)) {
            System.err
                    .println("Warning: init properties ignored, connection pool"
                            + " already in use.");
        }
    }

    /**
     * 
     * 
     * @return -1 if user aborted the login <br>
     *         0 if the user has logged-in but we should download observing
     *         runs, etc. <br>
     *         +1 if the user has logged-in and nothing more needs to be done.
     * @exception ObjectIOException
     *                Object I/O Exception
     */
    public static int login(JFrame frm, StorageManager dbaseMgr)
            throws ObjectIOException {
        String t = OHSApplication.getShortDesc() + " login";
        LoginDialog loginDialog = new LoginDialog(frm, t);
        AppConfig appcfg = AppConfig.getAppConfig();
        Config cfg = Config.getCfg();

        try {
            for (;;) {
                loginDialog.show();
                if (loginDialog.getUserChoice() == JOptionPane.OK_OPTION) {
                    int userID = loginDialog.getUserName();
                    if (userID == -1) {
                        JOptionPane.showMessageDialog(null,
                                "Username must be one or more digits",
                                "Login Error", JOptionPane.WARNING_MESSAGE);
                        continue;
                    }

                    cfg.setUserName(userID);
                    cfg.setPassword(loginDialog.getPassword());
                    dbaseMgr.setUserId(cfg.getUserName());

                    if (appcfg.isSuperUserEntitled()
                            || dbaseMgr.loginUser(cfg.getPassword())) {
                        loginDialog.dispose();
                        if (cfg.isInOperationsMode()) {
                            appcfg.setSuperUserEntitled(true);
                        }
                        return 0;
                    } else {
                        JOptionPane.showMessageDialog(null,
                                "Invalid username or password", "Login Error",
                                JOptionPane.INFORMATION_MESSAGE);
                    }
                } else {
                    return -1;
                }
            }
        } catch (ObjectIOException e) {
            stdlog_.error("", e);
            ; // Leave as user support info.
            JOptionPane.showMessageDialog(null, new Object[]
            {
                    "Your username and password could not be verified"
                            + " This is usually because",
                    "the server at ESO cannot be contacted, for some reason.",
                    " ", "Check your connection to ESO" + " and try again.",
                    "If error persists, contact ESO User Support.", " ",
                    "Fatal error during initialisation" }, "Fatal Error",
                    JOptionPane.ERROR_MESSAGE);
            System.exit(1);
        }

        return -1;
    }

    public static StorageManager createStorageManager(Media device)
            throws ObjectIOException {
        AppConfig appcfg = AppConfig.getAppConfig();

        /*
         * if (appcfg.shouldUseAppServer()) { return new
         * AuthenticatingStorageMgr( new RmiLocalManager(device,
         * appcfg.getAppServerUrl())
         *  ); } else
         */{
            StorageManager stMgr;

            if (device.equals(Media.PHASE1_DB)) {
                stMgr = new OpcDbaseMgr();
            } else if (device.equals(Media.PHASE2_DB)) {
                stMgr = new DbaseMgr();
            } else {
                throw new IllegalArgumentException("Unknown device: " + device);
            }

            return new AuthenticatingStorageMgr(new ProtectingStorageMgr(stMgr));
        }
    }

    public static void showCcsFailureErrMsg(Throwable e) {
        showCcsFailureErrMsg("No CCS connection available."
                + "\nPossible reasons for connection failure include:"
                + "\n  - site.cf configuration file missing or incorrect."
                + "\n  - RTAPENV environment variable missing or wrong."
                + "\n  - Another CCS process with the same name" + "\n    '"
                + AppConfig.getAppConfig().getCCSServer()
                + "' is already subscribed.", e);
    }

    public static void showCcsFailureErrMsg(String msg, Throwable e) {
        JOptionPane.showMessageDialog(null,
        /* MainWindow.getPrincipalWindow(), */
        new Object[]
        { msg, "Error Message: " + e.getMessage() }, "No connection to BOB",
                JOptionPane.WARNING_MESSAGE);
    }

    public static void main(String[] args) {
        OT ot = new OT(args);
    }

}