package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.ReadmeAttribute;
import org.eso.ohs.instruments.ReadmeAttributesFactory;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
import org.eso.ohs.phase2.apps.p2pp.ReadmeEditView;

/**
 * Action class to check-in the selected readmee.
 * 
 * <p>
 * <br>
 * 
 * @author Bob Kemp <br>
 *         ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP
 *         AppServer. <br>
 *         ChangeLog: cmavros 28JUNE02 Correct the validation of OBs through
 *         P2PP AppServer.
 */
public class CheckInReadmeAction extends ActionSuperclass {

    /*
    * Status window which displays reports on imports.     
    */

   private static TextDisplayWidget log_ =
       new TextDisplayWidget("Readme Verification Report");


    /** FolderGridView for the folder containing the selected OB */
    private HierarchyView hView_;

    private Logger stdlog_ = (Logger) Logger.getInstance(VerifyReadmeFileAction.class);

    

    /**
     * Constructs a check-in action.
     */
    public CheckInReadmeAction(HierarchyView hView) {
        super((JComponent)hView,"CheckIn Readme");
        hView_ = hView;
    }

    public void actionPerformedImpl(ActionEvent ev) {
        Persistence objMgr = Persistence.getInstance();
        ObservingRun or=null;
        if(!OHSApplication.ensureAuthenticated())
             return;
        try {
            or = hView_.getSelectedFolder().findObsRun();
        } catch (ObjectIOException e1) {
            ErrorMessages.announceIOError(hView_,e1);
        }
        if (!or.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_STANDARD)) {
            JOptionPane.showMessageDialog(hView_.getTopLevelAncestor(),
                    new Object[] {
                            "Only readme files for service-mode Observing Runs",
                            "can be checked-in." }, "Cannot Check-In",
                    JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        Readme readme = or.getReadme();
        if(readme != null) {
            readme.setStatus(Readme.DEFINED_STATUS);
            if(or.getIPVersion()!=or.getReadme().getIpVersion()) {
                Instrument inst = InstrumentList.getInstance().getInstrument(or.getInstCode(),or.getIPVersion());
                boolean isEquals = ReadmeAttributesFactory.isEqualsMeta(inst,readme.getRedmeAttributes());
                ReadmeAttribute [] newReadmeAttributes = ReadmeAttributesFactory.refreshReadmeAttributes(inst,readme.getRedmeAttributes());
                readme.setRedmeAttributes(newReadmeAttributes);
                readme.setIpVersion(or.getIPVersion());
                if(!isEquals) {
                    JOptionPane.showMessageDialog(
                        hView_,
                        new Object[] {
                            "This Readme has been updated to use",
                            "the Instrument Package specified in the Observing Run." },
                        "Update to new Instrument Package",
                        JOptionPane.INFORMATION_MESSAGE);
                }
                // fire event to save new ip version
                or.setReadme(readme);

            }
            log_.setSize(new Dimension(700, 300));
            log_.clearTextArea();
            String report = readme.verifyReport();
            if(report.length() > 0) {                          
                log_.setTextArea("Readme failed verification and cannot be checked in \n" + report.toString());
                log_.setVisible(true);
                return;
                
            }
            if(readme.isCheckedIn()) {
                JOptionPane.showMessageDialog(hView_.getTopLevelAncestor(),
                        new Object[] {
                                "Readme is already checked in",
                                "Please check the readme out make changes and the check the readme" }, "Cannot Check-In",
                        JOptionPane.ERROR_MESSAGE);
                return;
                
            }                        
         } else {
             JOptionPane.showMessageDialog(
                     hView_,
                     new Object[] {
                         "Readme file not created yet for this observing run"
                         },
                     "Readme Not Created",
                     JOptionPane.ERROR_MESSAGE);
             return;        
         }

        int option = JOptionPane
                .showConfirmDialog(
                        hView_.getTopLevelAncestor(),
                        new Object[] { "Are you sure you want to check-in the readme file for this run ?" },
                        "Continue with Check-In?", JOptionPane.OK_CANCEL_OPTION);
        switch (option) {
        case JOptionPane.CLOSED_OPTION:
            return;
        case JOptionPane.CANCEL_OPTION:
            return;
        }
        Persistence persistence = Persistence.getInstance();
        try {
            persistence.checkInReadmeFile(or);
            readme.setCheckedIn(true);
            MainWindow.getPrincipalWindow().setReadmeIcon(false);
            or.setReadme(readme);
            if(ReadmeEditView.getInstance().isVisible())
                ReadmeEditView.getInstance().init(or);
            
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(hView_,e);
        } catch (ObjectNotFoundException e) {
            ErrorMessages.announceNoObject(hView_,e);
        } catch (IOException e) {
            ErrorMessages.announceIOError(hView_,e);
        }
    }
}