package org.eso.ohs.phase2.visibility;

/**
 * Utility class for converting a PCF to a Dataset.
 *
 * @author Nick Kornweibel for ESO
 */

import java.util.Date;

import javachart.chart.Dataset;

import org.apache.log4j.Logger;

public class PCF2Dataset {

   /**
    * Standard log channel
    * <br>ChangeLog: cmavros 21-12-01 added
    */
   private static Logger stdlog_ =
   Logger.getLogger( PCF2Dataset.class );
  
    /**
     * Extract the data from the PCF and return it in a Dataset.
     * Start and end time are taken from the PCF.
     */

    public static Dataset convertPCF2Dataset(PCF pcf, int precision){
        Dataset rc = new Dataset();
        double vals[] = convertPCF2YAxisValues(pcf, precision);
        // Move the data taken from the PCF into the Dataset.
        for (int i = 0; i < vals.length; i++) {
            rc.addPoint((double)i, vals[i], "");
        }
        return rc;
    }

    /**
     * Extract the values from the PCF (from start time to end time in
     * the PCF) and return it as an array of doubles representing the
     * Y Axis Values in a graph.
     */

    public static double[] convertPCF2YAxisValues(PCF pcf, int precision){
        // create an empty dataset.
        double[] vals = null;

        // there is only something to plot if there are more than
        // 2 intervals in the PCF, as the -ve inf and +inf intervals
        // are of no interest to us.

        if (pcf.getIntervalCount() > 2) {
            // This assumes that the PCF covers -ve infinity to +ve
            // infinity in its bounds....which it should!

            PCFInterval startInterval = null;
            PCFInterval endInterval = null;
            boolean emptyPlot = false;

            int idx = 1;
            do {
                startInterval = pcf.getIntervalAt(idx++);
            } while (PCF.dateIsNegativeInfinity(startInterval.getStart()));

            if (PCF.dateIsPositiveInfinity(startInterval.getStart())) {
                emptyPlot = true;
            }

            idx = pcf.getIntervalCount()-1;
            do {
                endInterval = pcf.getIntervalAt(idx--);
            } while (PCF.dateIsPositiveInfinity(endInterval.getStart()));

            if (PCF.dateIsNegativeInfinity(endInterval.getStart())) {
                emptyPlot = true;
            }

            if (!emptyPlot) {
                Date startDate = startInterval.getStart();
                Date endDate = endInterval.getStart();
                //stdlog_.info("Start Date: <"+startDate +">");
                //stdlog_.info("End Date: <"+endDate +">");
                //stdlog_.info("PCF:\n"+pcf.toString());
                long start = startDate.getTime();
                long end = endDate.getTime();
                long milliPrecision = 1000*60*precision;
                int valCount = (int)((end-start)/milliPrecision)+1;
                if (valCount > 40000) {
                    throw new ArrayIndexOutOfBoundsException("pcf values too many:"+ valCount);
                }
                vals = new double[valCount];
                for (int i = 0; i < valCount; i++) {
                    Date nowDate = new Date(start+(milliPrecision*i));
                    vals[i] = pcf.getValueAt(nowDate);
                    //stdlog_.info(nowDate+" Val: "+i+" "+vals[i]);
                }
            } else {
                // no useful data in this PCF - create an empty plot
                vals = new double[0];
            }
        } else {
            vals = new double[0];
        } 
        return vals;
    }
}
