package org.eso.ohs.phase2.visiplot;

/**
 * @version $Revision: 1.10 $
 * @author  Nick Kornweibel for ESO. Created on July 31, 2001, 10:45 AM
 * <br>ChangeLog: cmavros 26/09/01 Add Time Interval menu option 
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.eso.ohs.core.utilities.Convert;

import org.eso.ohs.phase2.visibility.AirMass;

public class PlotOptionsPanel extends JPanel {

	private JLabel startTimeLabel_;

	private JTextField startTimeEdit_;

	private JLabel endTimeLabel_;

	private JTextField endTimeEdit_;

	private JCheckBox groupIdentOBsCheckBox_;

	private JCheckBox airMassConCheckBox_;

	private JTextField airMassConEdit_;

	private JCheckBox moonDistConCheckBox_;

	private JTextField moonDistConEdit_;

	private JCheckBox fliConCheckBox_;

	private JTextField fliConEdit_;

	private JCheckBox sunCheckBox_;

	private JTextField sunConEdit_;

	private JCheckBox execTimeCheckBox_;

	private JCheckBox timeIntervalCheckBox_;

	private JCheckBox siderealTimeIntervalCheckBox_;

	private JLabel columnHolder_;
	
	private JCheckBox zenithConstraintCheckBox_;
		

	/**
	 * Creates new form PlotOptionsPanel
	 */
	
	public PlotOptionsPanel(boolean disablePlotOptions) {
		initComponents (disablePlotOptions);
	}	
	
	/**
	 * This method is called from within the constructor to
	 * initialize the form.
	 */

	private void initComponents(boolean disablePlotOptions) {
		startTimeLabel_ = new JLabel();
		startTimeEdit_ = new JTextField();
		endTimeLabel_ = new JLabel();
		endTimeEdit_ = new JTextField();
		groupIdentOBsCheckBox_ = new JCheckBox();
		airMassConCheckBox_ = new JCheckBox();
		airMassConEdit_ = new JTextField();
		moonDistConCheckBox_ = new JCheckBox();
		moonDistConEdit_ = new JTextField();
		fliConCheckBox_ = new JCheckBox();
		fliConEdit_ = new JTextField();
		sunCheckBox_ = new JCheckBox();
		sunConEdit_ = new JTextField();
		execTimeCheckBox_ = new JCheckBox();
		columnHolder_ = new JLabel();
		timeIntervalCheckBox_ = new JCheckBox();
		siderealTimeIntervalCheckBox_ = new JCheckBox();
		zenithConstraintCheckBox_ = new JCheckBox();
		

		setLayout(new BorderLayout());
		JPanel northPanel = new JPanel();
		northPanel.setLayout(new GridLayout(3, 2, 10, 0));
		add(northPanel, BorderLayout.NORTH);
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new GridLayout(7, 2, 10, 0));
		add(centerPanel, BorderLayout.CENTER);

		centerPanel.setBorder(new TitledBorder("Constraints"));

		startTimeLabel_.setText("Start Time:");
		startTimeLabel_.setFont(new Font("Dialog", 1, 11));
		endTimeLabel_.setText("End Time (not implemented):");
		endTimeLabel_.setFont(new Font("Dialog", 1, 11));
		setEditable(endTimeEdit_, false);
		endTimeEdit_.setBackground(Color.gray);
		groupIdentOBsCheckBox_.setText("group identical OBs");

		northPanel.add(startTimeLabel_);
		northPanel.add(startTimeEdit_);
		northPanel.add(endTimeLabel_);
		northPanel.add(endTimeEdit_);
		northPanel.add(groupIdentOBsCheckBox_);

		airMassConCheckBox_.setText("air mass");
		airMassConCheckBox_.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				airMassConRadio_StateChanged(evt);
			}
		});

		centerPanel.add(airMassConCheckBox_);
		centerPanel.add(airMassConEdit_);
		moonDistConCheckBox_.setText("moon distance");
		moonDistConCheckBox_.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				moonDistConRadio_StateChanged(evt);
			}
		});

		centerPanel.add(moonDistConCheckBox_);
		centerPanel.add(moonDistConEdit_);

		fliConCheckBox_.setText("fractional lunar ill.");
		fliConCheckBox_.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				fliConRadio_StateChanged(evt);
			}
		});

		centerPanel.add(fliConCheckBox_);
		centerPanel.add(fliConEdit_);

		sunCheckBox_.setText("sun angle");
		sunCheckBox_.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				sunConRadio_StateChanged(evt);
			}
		});

		centerPanel.add(sunCheckBox_);
		centerPanel.add(sunConEdit_);

		timeIntervalCheckBox_.setText("time intervals");
		centerPanel.add(timeIntervalCheckBox_);

		// add empty space in RHS.
		centerPanel.add(columnHolder_);

		execTimeCheckBox_.setText("execution time");
		centerPanel.add(execTimeCheckBox_);

		centerPanel.add(new JLabel());
		siderealTimeIntervalCheckBox_.setText("st time intervals");
		centerPanel.add(siderealTimeIntervalCheckBox_);
		
		zenithConstraintCheckBox_.setText("zenith constraint");
		centerPanel.add(zenithConstraintCheckBox_);

		if (disablePlotOptions) {
			enableOptions(false);
		}

	}

	private void enableOptions( boolean enable) {
		startTimeEdit_.setEnabled(enable);
		endTimeEdit_.setEnabled(enable);
		//groupIdentOBsCheckBox_.setEnabled(enable);
		airMassConCheckBox_.setEnabled(enable);
		airMassConEdit_.setEnabled(enable);
		moonDistConCheckBox_.setEnabled(enable);
		moonDistConEdit_.setEnabled(enable);
		fliConCheckBox_.setEnabled(enable);
		fliConEdit_.setEnabled(enable);
		sunCheckBox_.setEnabled(enable);
		sunConEdit_.setEnabled(enable);
		execTimeCheckBox_.setEnabled(enable);
		timeIntervalCheckBox_.setEnabled(enable);
		siderealTimeIntervalCheckBox_.setEnabled(enable);	
		zenithConstraintCheckBox_.setEnabled(enable);
		
	}

	private void sunConRadio_StateChanged(ChangeEvent evt) {
		setEditable(sunConEdit_, sunCheckBox_.isSelected());
	}

	/**
	 * Set the start time edit field.
	 */

	public void setStartTime(String startTime) {
		startTimeEdit_.setText(startTime);
	}

	/**
	 * @return the start time string.
	 */

	public String getStartTime() {
		return startTimeEdit_.getText();
	}

	/**
	 * @return the end time string.
	 */

	public String getEndTime() {
		return endTimeEdit_.getText();
	}

	/**
	 * Set the end time edit field.
	 */

	public void setEndTime(String endTime) {
		endTimeEdit_.setText(endTime);
	}

	/**
	 * Set the radio button and text for FLI
	 * @param en true if the radio is to be selected
	 * @param value to be set to the edit area - null will result in no change.
	 */

	public void setSunConstraint(boolean en, String value) {
		sunCheckBox_.setSelected(en);
		setEditable(sunConEdit_, en);
		if (value != null) {
			sunConEdit_.setText(value);
		}
	}

	/**
	 * @return true if this constraint is selected
	 */

	public boolean sunConstraintSelected() {
		return sunCheckBox_.isSelected();
	}

	/**
	 * @return the value of the constraint parameter
	 */

	public String getSunConstraint() {
		return sunConEdit_.getText();
	}

	private void fliConRadio_StateChanged(ChangeEvent evt) {
		setEditable(fliConEdit_, fliConCheckBox_.isSelected());
	}

	/**
	 * Set the radio button and text for FLI
	 * @param en true if the radio is to be selected
	 * @param value to be set to the edit area - null will result in no change.
	 */

	public void setFLIConstraint(boolean en, String value) {
		fliConCheckBox_.setSelected(en);
		setEditable(fliConEdit_, en);
		if (value != null) {
			fliConEdit_.setText(value);
		}
	}

	/**
	 * @return true if this constraint is selected
	 */

	public boolean fliConstraintSelected() {
		return fliConCheckBox_.isSelected();
	}

	/**
	 * @return the value of the constraint parameter
	 */

	public String getFLIConstraint() {
		return fliConEdit_.getText();
	}

	private void moonDistConRadio_StateChanged(ChangeEvent evt) {
		setEditable(moonDistConEdit_, moonDistConCheckBox_.isSelected());
	}

	/**
	 * Set the radio button and text for Moon Distance
	 * @param en true if the radio is to be selected
	 * @param value to be set to the edit area - null will result in no change.
	 */

	public void setMoonDistConstraint(boolean en, String value) {
		moonDistConCheckBox_.setSelected(en);
		setEditable(moonDistConEdit_, en);
		if (value != null) {
			moonDistConEdit_.setText(value);
		}
	}

	/**
	 * @return true if this constraint is selected
	 */

	public boolean moonDistConstraintSelected() {
		return moonDistConCheckBox_.isSelected();
	}
	
	
	public boolean zenithConstraintSelected() {
		return zenithConstraintCheckBox_.isSelected();
	}
	
	public void setZenithConstraint(boolean enabled) {
		zenithConstraintCheckBox_.setSelected(enabled);
	}

	/**
	 * @return the value of the constraint parameter
	 */

	public String getMoonDistConstraint() {
		return moonDistConEdit_.getText();
	}

	private void airMassConRadio_StateChanged(ChangeEvent evt) {
		setEditable(airMassConEdit_, airMassConCheckBox_.isSelected());
	}

	/**
	 * Set the radio button and text for Air Mass
	 * @param en true if the radio is to be selected
	 * @param value to be set to the edit area - null will result in no change.
	 */

	public void setAirMassConstraint(boolean en, String value) {
		airMassConCheckBox_.setSelected(en);
		setEditable(airMassConEdit_, en);
		if (value != null) {
			airMassConEdit_.setText(value);
		}
	}

	/**
	 * @return true if this constraint is selected
	 */

	public boolean airMassConstraintSelected() {
		return airMassConCheckBox_.isSelected();
	}

	/**
	 * @return the value of the constraint parameter
	 */

	public String getAirMassConstraint() {
		return airMassConEdit_.getText();
	}

	/**
	 * Set the radio button and text for Exec Time
	 * @param en true if the radio is to be selected
	 */

	public void setExecTimeConstraint(boolean en) {
		execTimeCheckBox_.setSelected(en);
	}

	/**
	 * @return true if the exec time constraint is selected
	 */

	public boolean execTimeConstraintSelected() {
		return execTimeCheckBox_.isSelected();
	}

	/**
	 * Set the radio button and text for Time Interval
	 * @param en true if the radio is to be selected
	 */

	public void setTimeIntervalConstraint(boolean en) {
		timeIntervalCheckBox_.setSelected(en);
	}

	/**
	 * @return true if the Time Interval constraint is selected
	 */

	public boolean timeIntervalConstraintSelected() {
		return timeIntervalCheckBox_.isSelected();
	}

	/**
	 * Set the radio button and text for Sidereal Time Interval
	 * @param en true if the radio is to be selected
	 */

	public void setSiderealTimeIntervalConstraint(boolean en) {
		siderealTimeIntervalCheckBox_.setSelected(en);
	}

	/**
	 * @return true if the Sidereal Time Interval constraint is selected
	 */

	public boolean siderealTimneIntervalConstraintSelected() {
		return siderealTimeIntervalCheckBox_.isSelected();
	}

	/**
	 * Set the radio button and text for Exec Time
	 * @param en true if the radio is to be selected
	 */

	public void setGroupIdentOBsConstraint(boolean en) {
		groupIdentOBsCheckBox_.setSelected(en);
	}

	/**
	 * @return true if "group identical obs" is selected
	 */

	public boolean groupIdentOBsConstraintSelected() {
		return groupIdentOBsCheckBox_.isSelected();
	}

	/**
	 * Check all entries in the panel. Empty text fields are set to
	 * VisiPlotOptions.readFromOB.
	 * @return a report on the validity of the entries in the panel, ""
	 * indicates that there is no problem with the entries.
	 */

	public String checkEntries() {
		StringBuffer rc = new StringBuffer("");
		SimpleDateFormat formatter = new SimpleDateFormat(
				Convert.ISO_DATE_FORMAT);
		Date startTime = formatter.parse(startTimeEdit_.getText(),
				new ParsePosition(0));
		if (startTime == null) {
			rc.append("Start time is not ISO format: "
					+ Convert.ISO_DATE_FORMAT + "\n");
		}

		Date endTime = formatter.parse(endTimeEdit_.getText(),
				new ParsePosition(0));
		if (endTime == null) {
			rc.append("End time is not ISO format: " + Convert.ISO_DATE_FORMAT);
		}

		// airMassConEdit_
		if (getAirMassConstraint().equals("")) {
			setAirMassConstraint(airMassConstraintSelected(),
					VisiPlotOptions.useOBValue);
		}
		if (airMassConstraintSelected()) {
			String val = getAirMassConstraint();
			if (!(VisiPlotOptions.isReadFromOB(val) || VisiPlotOptions
					.validDouble(val, 1.0, AirMass.NOT_VIS))) {
				rc.append("Air Mass must be a number between " + "1.0 and "
						+ AirMass.NOT_VIS + "\n");
			}
		}
		// moonDistConEdit_
		if (getMoonDistConstraint().equals("")) {
			setMoonDistConstraint(moonDistConstraintSelected(),
					VisiPlotOptions.useOBValue);
		}
		if (moonDistConstraintSelected()) {
			String val = getMoonDistConstraint();
			if (!(VisiPlotOptions.isReadFromOB(val) || VisiPlotOptions
					.validDouble(val, 0.0, 180.0))) {
				rc
						.append("Moon Distance must be a number between 0.0 and 180.0.\n");
			}
		}
		// fliConEdit_
		if (getFLIConstraint().equals("")) {
			setFLIConstraint(fliConstraintSelected(),
					VisiPlotOptions.useOBValue);
		}
		if (fliConstraintSelected()) {
			String val = getFLIConstraint();
			if (!(VisiPlotOptions.isReadFromOB(val) || VisiPlotOptions
					.validDouble(val, 0.0, 1.0))) {
				rc
						.append("Fraction Lunar Illum. must be a number between 0.0 and 1.0.\n");
			}
		}
		// sunConEdit_
		if (getSunConstraint().equals("")) {
			setSunConstraint(sunConstraintSelected(),
					VisiPlotOptions.defaultSunCon);
		}
		if (sunConstraintSelected()) {
			String val = getSunConstraint();
			if (!(VisiPlotOptions.isReadFromOB(val) || VisiPlotOptions
					.validInteger(val, -25, 0))) {
				rc
						.append("Sunset angle must be an integer between -25 and 0.\n");
			}
		}
		return rc.toString();
	}

	private void setEditable(JTextField t, boolean ed) {
		t.setEditable(ed);
		//t.setEnabled(ed);
		if (ed) {
			t.setBackground(Color.white);
		} else {
			t.setBackground(Color.gray);
		}
	}
}
